@@ -1,19 +1,30 @@
 """Provide strategies for given endpoint(s) definition."""
 import asyncio
-from typing import Any, Callable, List, Optional, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 import hypothesis
 from hypothesis import strategies as st
+from hypothesis.strategies import SearchStrategy
+from hypothesis.utils.conventions import InferType
 
-from .constants import DEFAULT_DEADLINE
+from .constants import DEFAULT_DEADLINE, DataGenerationMethod
 from .exceptions import InvalidSchema
 from .hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher
 from .models import Case, Endpoint
 from .stateful import Feedback, Stateful
 
+GivenInput = Union[SearchStrategy, InferType]
+
 
 def create_test(
-    endpoint: Endpoint, test: Callable, settings: Optional[hypothesis.settings] = None, seed: Optional[int] = None
+    *,
+    endpoint: Endpoint,
+    test: Callable,
+    settings: Optional[hypothesis.settings] = None,
+    seed: Optional[int] = None,
+    data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
+    _given_args: Tuple[GivenInput, ...] = (),
+    _given_kwargs: Optional[Dict[str, GivenInput]] = None,
 ) -> Callable:
     """Create a Hypothesis test."""
     hook_dispatcher = getattr(test, "_schemathesis_hooks", None)
@@ -22,8 +33,12 @@ def create_test(
         feedback = Feedback(endpoint.schema.stateful, endpoint)
     else:
         feedback = None
-    strategy = endpoint.as_strategy(hooks=hook_dispatcher, feedback=feedback)
-    wrapped_test = hypothesis.given(case=strategy)(test)
+    strategy = endpoint.as_strategy(
+        hooks=hook_dispatcher, feedback=feedback, data_generation_method=data_generation_method
+    )
+    _given_kwargs = (_given_kwargs or {}).copy()
+    _given_kwargs.setdefault("case", strategy)
+    wrapped_test = hypothesis.given(*_given_args, **_given_kwargs)(test)
     if seed is not None:
         wrapped_test = hypothesis.seed(seed)(wrapped_test)
     if asyncio.iscoroutinefunction(test):
@@ -39,16 +54,22 @@ def setup_default_deadline(wrapped_test: Callable) -> None:
     # Quite hacky, but it is the simplest way to set up the default deadline value without affecting non-Schemathesis
     # tests globally
     existing_settings = getattr(wrapped_test, "_hypothesis_internal_use_settings", None)
-    if existing_settings.deadline == hypothesis.settings.default.deadline:
+    if existing_settings is not None and existing_settings.deadline == hypothesis.settings.default.deadline:
         new_settings = hypothesis.settings(existing_settings, deadline=DEFAULT_DEADLINE)
         wrapped_test._hypothesis_internal_use_settings = new_settings  # type: ignore
 
 
 def make_test_or_exception(
-    endpoint: Endpoint, func: Callable, settings: Optional[hypothesis.settings] = None, seed: Optional[int] = None
+    endpoint: Endpoint,
+    func: Callable,
+    settings: Optional[hypothesis.settings] = None,
+    seed: Optional[int] = None,
+    data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
 ) -> Union[Callable, InvalidSchema]:
     try:
-        return create_test(endpoint, func, settings, seed=seed)
+        return create_test(
+            endpoint=endpoint, test=func, settings=settings, seed=seed, data_generation_method=data_generation_method
+        )
     except InvalidSchema as exc:
         return exc
 
