@@ -1,6 +1,7 @@
+import os
 import re
 from contextlib import contextmanager
-from typing import Dict, Generator, Optional, Tuple
+from typing import Dict, Generator, Optional, Tuple, Union
 from urllib.parse import urlparse
 
 import click
@@ -9,11 +10,16 @@ from requests import PreparedRequest, RequestException
 
 from .. import utils
 from ..stateful import Stateful
+from .constants import DEFAULT_WORKERS
 
 
 def validate_schema(ctx: click.core.Context, param: click.core.Parameter, raw_value: str) -> str:
     if "app" not in ctx.params:
-        if not urlparse(raw_value).netloc:
+        try:
+            netloc = urlparse(raw_value).netloc
+        except ValueError as exc:
+            raise click.UsageError("Invalid SCHEMA, must be a valid URL or file path.") from exc
+        if not netloc:
             if "\x00" in raw_value or not utils.file_exists(raw_value):
                 raise click.UsageError("Invalid SCHEMA, must be a valid URL or file path.")
             if "base_url" not in ctx.params:
@@ -31,11 +37,23 @@ def _validate_url(value: str) -> None:
 
 
 def validate_base_url(ctx: click.core.Context, param: click.core.Parameter, raw_value: str) -> str:
-    if raw_value and not urlparse(raw_value).netloc:
+    try:
+        netloc = urlparse(raw_value).netloc
+    except ValueError as exc:
+        raise click.UsageError("Invalid base URL") from exc
+    if raw_value and not netloc:
         raise click.UsageError("Invalid base URL")
     return raw_value
 
 
+APPLICATION_FORMAT_MESSAGE = (
+    "Can not import application from the given module!\n"
+    "The `--app` option value should be in format:\n\n    path:variable\n\n"
+    "where `path` is an importable path to a Python module,\n"
+    "and `variable` is a variable name inside that module."
+)
+
+
 def validate_app(ctx: click.core.Context, param: click.core.Parameter, raw_value: Optional[str]) -> Optional[str]:
     if raw_value is None:
         return raw_value
@@ -47,11 +65,11 @@ def validate_app(ctx: click.core.Context, param: click.core.Parameter, raw_value
         return raw_value
     except Exception as exc:
         show_errors_tracebacks = ctx.params["show_errors_tracebacks"]
-        message = utils.format_exception(exc, show_errors_tracebacks)
-        click.secho(f"{message}\nCan not import application from the given module", fg="red")
+        message = utils.format_exception(exc, show_errors_tracebacks).strip()
+        click.secho(f"{APPLICATION_FORMAT_MESSAGE}\n\nException:\n\n{message}", fg="red")
         if not show_errors_tracebacks:
             click.secho(
-                "Add this option to your command line parameters to see full tracebacks: --show-errors-tracebacks",
+                "\nAdd this option to your command line parameters to see full tracebacks: --show-errors-tracebacks",
                 fg="red",
             )
         raise click.exceptions.Exit(1)
@@ -98,7 +116,7 @@ def validate_regex(ctx: click.core.Context, param: click.core.Parameter, raw_val
     for value in raw_value:
         try:
             re.compile(value)
-        except re.error as exc:
+        except (re.error, OverflowError, RuntimeError) as exc:
             raise click.BadParameter(f"Invalid regex: {exc.args[0]}")
     return raw_value
 
@@ -117,9 +135,35 @@ def convert_stateful(ctx: click.core.Context, param: click.core.Parameter, value
     return Stateful[value]
 
 
+def convert_request_tls_verify(ctx: click.core.Context, param: click.core.Parameter, value: str) -> Union[str, bool]:
+    if value.lower() in ("y", "yes", "t", "true", "on", "1"):
+        return True
+    if value.lower() in ("n", "no", "f", "false", "off", "0"):
+        return False
+    return value
+
+
 @contextmanager
 def reraise_format_error(raw_value: str) -> Generator[None, None, None]:
     try:
         yield
     except ValueError as exc:
         raise click.BadParameter(f"Should be in KEY:VALUE format. Got: {raw_value}") from exc
+
+
+def get_workers_count() -> int:
+    """Detect the number of available CPUs for the current process, if possible.
+
+    Use ``DEFAULT_WORKERS`` if not possible to detect.
+    """
+    if hasattr(os, "sched_getaffinity"):
+        # In contrast with `os.cpu_count` this call respects limits on CPU resources on some Unix systems
+        return len(os.sched_getaffinity(0))
+    # Number of CPUs in the system, or 1 if undetermined
+    return os.cpu_count() or DEFAULT_WORKERS
+
+
+def convert_workers(ctx: click.core.Context, param: click.core.Parameter, value: str) -> int:
+    if value == "auto":
+        return get_workers_count()
+    return int(value)
