@@ -14,21 +14,28 @@ from ...runner.serialization import SerializedCase, SerializedTestResult
 from ..context import ExecutionContext
 from ..handlers import EventHandler, get_unique_failures
 
+DISABLE_SCHEMA_VALIDATION_MESSAGE = (
+    "\nYou can disable input schema validation with --validate-schema=false "
+    "command-line option\nIn this case, Schemathesis cannot guarantee proper"
+    " behavior during the test run"
+)
+
 
 def get_terminal_width() -> int:
     return shutil.get_terminal_size().columns
 
 
-def display_section_name(title: str, separator: str = "=", **kwargs: Any) -> None:
+def display_section_name(title: str, separator: str = "=", extra: str = "", **kwargs: Any) -> None:
     """Print section name with separators in terminal with the given title nicely centered."""
-    message = f" {title} ".center(get_terminal_width(), separator)
+    extra = extra if not extra else f" [{extra}]"
+    message = f" {title}{extra} ".center(get_terminal_width(), separator)
     kwargs.setdefault("bold", True)
     click.secho(message, **kwargs)
 
 
 def display_subsection(result: SerializedTestResult, color: Optional[str] = "red") -> None:
     section_name = f"{result.method}: {result.path}"
-    display_section_name(section_name, "_", fg=color)
+    display_section_name(section_name, "_", result.data_generation_method, fg=color)
 
 
 def get_percentage(position: int, length: int) -> str:
@@ -126,6 +133,8 @@ def display_single_error(context: ExecutionContext, result: SerializedTestResult
             message = error.exception_with_traceback
         else:
             message = error.exception
+        if error.exception.startswith("schemathesis.exceptions.InvalidSchema"):
+            message += DISABLE_SCHEMA_VALIDATION_MESSAGE + "\n"
         click.secho(message, fg="red")
         if error.example is not None:
             display_example(context, error.example, seed=result.seed)
@@ -155,7 +164,7 @@ def display_failures_for_single_test(context: ExecutionContext, result: Serializ
             message = f"{idx}. {check.message}"
         else:
             message = None
-        example = cast(SerializedCase, check.example)  # filtered in `_get_unique_failures`
+        example = cast(SerializedCase, check.example)  # filtered in `get_unique_failures`
         display_example(context, example, message, result.seed)
         # Display every time except the last check
         if idx != len(checks):
@@ -185,9 +194,9 @@ def display_example(
         click.secho(line, fg="red")
     if case.text_lines:
         click.echo()
-    click.secho(f"Run this Python code to reproduce this failure: \n\n    {case.requests_code}", fg="red")
+    click.secho(f"Run this Python code to reproduce this failure: \n\n    {case.requests_code}\n", fg="red")
     if seed is not None:
-        click.secho(f"\nOr add this option to your command line parameters: --hypothesis-seed={seed}", fg="red")
+        click.secho(f"Or add this option to your command line parameters: --hypothesis-seed={seed}", fg="red")
 
 
 def display_application_logs(context: ExecutionContext, event: events.Finished) -> None:
@@ -265,11 +274,7 @@ def display_internal_error(context: ExecutionContext, event: events.InternalErro
             f"Add this option to your command line parameters to see full tracebacks: --show-errors-tracebacks"
         )
         if event.exception_type == "jsonschema.exceptions.ValidationError":
-            message += (
-                "\n\nYou can disable input schema validation with --validate-schema=false "
-                "command-line option\nIn this case, Schemathesis cannot guarantee proper"
-                " behavior during the test run"
-            )
+            message += "\n" + DISABLE_SCHEMA_VALIDATION_MESSAGE
         click.secho(message, fg="red")
 
 
