@@ -1,5 +1,5 @@
 from functools import partial
-from typing import Any, Callable, Generator, List, Optional, Type, Union, cast
+from typing import Any, Callable, Generator, List, Optional, Type, TypeVar, Union, cast
 
 import pytest
 from _pytest import fixtures, nodes
@@ -9,9 +9,10 @@ from _pytest.nodes import Node
 from _pytest.python import Class, Function, FunctionDefinition, Metafunc, Module, PyCollector
 from _pytest.runner import runtestprotocol
 from _pytest.warning_types import PytestWarning
-from hypothesis.errors import InvalidArgument  # pylint: disable=ungrouped-imports
+from hypothesis.errors import InvalidArgument
 from packaging import version
 
+from .. import DataGenerationMethod
 from .._hypothesis import create_test
 from ..exceptions import InvalidSchema
 from ..models import Endpoint
@@ -20,23 +21,102 @@ from ..utils import is_schemathesis_test
 
 USE_FROM_PARENT = version.parse(pytest.__version__) >= version.parse("5.4.0")
 
+T = TypeVar("T", bound=Node)
 
-def create(cls: Type[Node], *args: Any, **kwargs: Any) -> Node:
+
+def create(cls: Type[T], *args: Any, **kwargs: Any) -> T:
     if USE_FROM_PARENT:
-        return cls.from_parent(*args, **kwargs)
+        return cls.from_parent(*args, **kwargs)  # type: ignore
     return cls(*args, **kwargs)
 
 
+class SchemathesisFunction(Function):  # pylint: disable=too-many-ancestors
+    def __init__(
+        self,
+        *args: Any,
+        test_func: Callable,
+        test_name: Optional[str] = None,
+        recursion_level: int = 0,
+        data_generation_method: DataGenerationMethod,
+        **kwargs: Any,
+    ) -> None:
+        super().__init__(*args, **kwargs)
+        self.test_function = test_func
+        self.test_name = test_name
+        self.recursion_level = recursion_level
+        self.data_generation_method = data_generation_method
+
+    def _getobj(self) -> partial:
+        """Tests defined as methods require `self` as the first argument.
+
+        This method is called only for this case.
+        """
+        return partial(self.obj, self.parent.obj)  # type: ignore
+
+    @property
+    def feedback(self) -> Optional[Feedback]:
+        return getattr(self.obj, "_schemathesis_feedback", None)
+
+    def warn_if_stateful_responses_not_stored(self) -> None:
+        feedback = self.feedback
+        if feedback is not None and not feedback.stateful_tests:
+            self.warn(PytestWarning(NOT_USED_STATEFUL_TESTING_MESSAGE))
+
+    def _get_stateful_tests(self) -> List["SchemathesisFunction"]:
+        feedback = self.feedback
+        recursion_level = self.recursion_level
+        if feedback is None or recursion_level >= feedback.endpoint.schema.stateful_recursion_limit:
+            return []
+        previous_test_name = self.test_name or f"{feedback.endpoint.method.upper()}:{feedback.endpoint.full_path}"
+
+        def make_test(
+            endpoint: Endpoint,
+            test: Union[Callable, InvalidSchema],
+            data_generation_method: DataGenerationMethod,
+            previous_tests: str,
+        ) -> "SchemathesisFunction":
+            test_name = f"{previous_tests} -> {endpoint.method.upper()}:{endpoint.full_path}"
+            return create(
+                self.__class__,
+                name=f"{self.originalname}[{test_name}][{self.data_generation_method.as_short_name()}]",
+                parent=self.parent,
+                callspec=getattr(self, "callspec", None),
+                callobj=test,
+                fixtureinfo=self._fixtureinfo,
+                keywords=self.keywords,
+                originalname=self.originalname,
+                test_func=self.test_function,
+                test_name=test_name,
+                recursion_level=recursion_level + 1,
+                data_generation_method=data_generation_method,
+            )
+
+        return [
+            make_test(endpoint, test, data_generation_method, previous_test_name)
+            for (endpoint, data_generation_method, test) in feedback.get_stateful_tests(self.test_function, None, None)
+        ]
+
+    def add_stateful_tests(self) -> None:
+        idx = self.session.items.index(self) + 1
+        tests = self._get_stateful_tests()
+        self.session.items[idx:idx] = tests
+        self.session.testscollected += len(tests)
+
+
 class SchemathesisCase(PyCollector):
     def __init__(self, test_function: Callable, *args: Any, **kwargs: Any) -> None:
         self.test_function = test_function
         self.schemathesis_case = test_function._schemathesis_test  # type: ignore
+        self.given_args = getattr(test_function, "_schemathesis_given_args", ())
+        self.given_kwargs = getattr(test_function, "_schemathesis_given_kwargs", {})
         super().__init__(*args, **kwargs)
 
-    def _get_test_name(self, endpoint: Endpoint) -> str:
-        return f"{self.name}[{endpoint.method}:{endpoint.full_path}]"
+    def _get_test_name(self, endpoint: Endpoint, data_generation_method: DataGenerationMethod) -> str:
+        return f"{self.name}[{endpoint.method.upper()}:{endpoint.full_path}][{data_generation_method.as_short_name()}]"
 
-    def _gen_items(self, endpoint: Endpoint) -> Generator[Function, None, None]:
+    def _gen_items(
+        self, endpoint: Endpoint, data_generation_method: DataGenerationMethod
+    ) -> Generator[SchemathesisFunction, None, None]:
         """Generate all items for the given endpoint.
 
         Could produce more than one test item if
@@ -45,11 +125,11 @@ class SchemathesisCase(PyCollector):
         This implementation is based on the original one in pytest, but with slight adjustments
         to produce tests out of hypothesis ones.
         """
-        name = self._get_test_name(endpoint)
-        funcobj = self._make_test(endpoint)
+        name = self._get_test_name(endpoint, data_generation_method)
+        funcobj = self._make_test(endpoint, data_generation_method)
 
         cls = self._get_class_parent()
-        definition = create(FunctionDefinition, name=self.name, parent=self.parent, callobj=funcobj)
+        definition: FunctionDefinition = create(FunctionDefinition, name=self.name, parent=self.parent, callobj=funcobj)
         fixturemanager = self.session._fixturemanager
         fixtureinfo = fixturemanager.getfixtureinfo(definition, funcobj, cls)
 
@@ -64,6 +144,7 @@ class SchemathesisCase(PyCollector):
                 fixtureinfo=fixtureinfo,
                 test_func=self.test_function,
                 originalname=self.name,
+                data_generation_method=data_generation_method,
             )
         else:
             fixtures.add_funcarg_pseudo_fixture_def(self.parent, metafunc, fixturemanager)
@@ -80,6 +161,7 @@ class SchemathesisCase(PyCollector):
                     keywords={callspec.id: True},
                     originalname=name,
                     test_func=self.test_function,
+                    data_generation_method=data_generation_method,
                 )
 
     def _get_class_parent(self) -> Optional[Type]:
@@ -89,7 +171,8 @@ class SchemathesisCase(PyCollector):
     def _parametrize(
         self, cls: Optional[Type], definition: FunctionDefinition, fixtureinfo: FuncFixtureInfo
     ) -> Metafunc:
-        module = self.getparent(Module).obj
+        parent = self.getparent(Module)
+        module = parent.obj if parent is not None else parent
         metafunc = Metafunc(definition, fixtureinfo, self.config, cls=cls, module=module)
         methods = []
         if hasattr(module, "pytest_generate_tests"):
@@ -100,9 +183,15 @@ class SchemathesisCase(PyCollector):
         self.ihook.pytest_generate_tests.call_extra(methods, {"metafunc": metafunc})
         return metafunc
 
-    def _make_test(self, endpoint: Endpoint) -> Callable:
+    def _make_test(self, endpoint: Endpoint, data_generation_method: DataGenerationMethod) -> Callable:
         try:
-            return create_test(endpoint, self.test_function)
+            return create_test(
+                endpoint=endpoint,
+                test=self.test_function,
+                _given_args=self.given_args,
+                _given_kwargs=self.given_kwargs,
+                data_generation_method=data_generation_method,
+            )
         except InvalidSchema as exc:
             message = exc.args[0]
             return lambda: pytest.fail(message)
@@ -111,7 +200,10 @@ class SchemathesisCase(PyCollector):
         """Generate different test items for all endpoints available in the given schema."""
         try:
             return [
-                item for endpoint in self.schemathesis_case.get_all_endpoints() for item in self._gen_items(endpoint)
+                item
+                for data_generation_method in self.schemathesis_case.data_generation_methods
+                for endpoint in self.schemathesis_case.get_all_endpoints()
+                for item in self._gen_items(endpoint, data_generation_method)
             ]
         except Exception:
             pytest.fail("Error during collection")
@@ -123,68 +215,6 @@ NOT_USED_STATEFUL_TESTING_MESSAGE = (
 )
 
 
-class SchemathesisFunction(Function):  # pylint: disable=too-many-ancestors
-    def __init__(
-        self, *args: Any, test_func: Callable, test_name: Optional[str] = None, recursion_level: int = 0, **kwargs: Any
-    ) -> None:
-        super().__init__(*args, **kwargs)
-        self.test_function = test_func
-        self.test_name = test_name
-        self.recursion_level = recursion_level
-
-    def _getobj(self) -> partial:
-        """Tests defined as methods require `self` as the first argument.
-
-        This method is called only for this case.
-        """
-        return partial(self.obj, self.parent.obj)
-
-    @property
-    def feedback(self) -> Optional[Feedback]:
-        return getattr(self.obj, "_schemathesis_feedback", None)
-
-    def warn_if_stateful_responses_not_stored(self) -> None:
-        feedback = self.feedback
-        if feedback is not None and not feedback.stateful_tests:
-            self.warn(PytestWarning(NOT_USED_STATEFUL_TESTING_MESSAGE))
-
-    def _get_stateful_tests(self) -> List["SchemathesisFunction"]:
-        feedback = self.feedback
-        recursion_level = self.recursion_level
-        if feedback is None or recursion_level >= feedback.endpoint.schema.stateful_recursion_limit:
-            return []
-        previous_test_name = self.test_name or f"{feedback.endpoint.method}:{feedback.endpoint.full_path}"
-
-        def make_test(
-            endpoint: Endpoint, test: Union[Callable, InvalidSchema], previous_tests: str
-        ) -> SchemathesisFunction:
-            test_name = f"{previous_tests} -> {endpoint.method}:{endpoint.full_path}"
-            return create(
-                self.__class__,
-                name=f"{self.originalname}[{test_name}]",
-                parent=self.parent,
-                callspec=getattr(self, "callspec", None),
-                callobj=test,
-                fixtureinfo=self._fixtureinfo,
-                keywords=self.keywords,
-                originalname=self.originalname,
-                test_func=self.test_function,
-                test_name=test_name,
-                recursion_level=recursion_level + 1,
-            )
-
-        return [
-            make_test(endpoint, test, previous_test_name)
-            for (endpoint, test) in feedback.get_stateful_tests(self.test_function, None, None)
-        ]
-
-    def add_stateful_tests(self) -> None:
-        idx = self.session.items.index(self) + 1
-        tests = self._get_stateful_tests()
-        self.session.items[idx:idx] = tests
-        self.session.testscollected += len(tests)
-
-
 @hookimpl(hookwrapper=True)  # type:ignore # pragma: no mutate
 def pytest_pycollect_makeitem(collector: nodes.Collector, name: str, obj: Any) -> Generator[None, Any, None]:
     """Switch to a different collector if the test is parametrized marked by schemathesis."""
