@@ -1,6 +1,5 @@
-# pylint: disable=too-many-arguments
 import pathlib
-from typing import IO, Any, Callable, Dict, Optional, Union
+from typing import IO, Any, Callable, Dict, Iterable, Optional, Union
 from urllib.parse import urljoin
 
 import jsonschema
@@ -10,8 +9,9 @@ from jsonschema import ValidationError
 from starlette.applications import Starlette
 from starlette.testclient import TestClient as ASGIClient
 from werkzeug.test import Client
+from yarl import URL
 
-from .constants import USER_AGENT
+from .constants import DEFAULT_DATA_GENERATION_METHODS, USER_AGENT, DataGenerationMethod
 from .exceptions import HTTPError
 from .hooks import HookContext, dispatch
 from .lazy import LazySchema
@@ -31,6 +31,9 @@ def from_path(
     *,
     app: Any = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Iterable[DataGenerationMethod] = DEFAULT_DATA_GENERATION_METHODS,
+    force_schema_version: Optional[str] = None,
 ) -> BaseOpenAPISchema:
     """Load a file from OS path and parse to schema instance."""
     with open(path) as fd:
@@ -44,6 +47,9 @@ def from_path(
             operation_id=operation_id,
             app=app,
             validate_schema=validate_schema,
+            skip_deprecated_endpoints=skip_deprecated_endpoints,
+            data_generation_methods=data_generation_methods,
+            force_schema_version=force_schema_version,
         )
 
 
@@ -54,13 +60,21 @@ def from_uri(
     endpoint: Optional[Filter] = None,
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
+    port: Optional[int] = None,
     *,
     app: Any = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Iterable[DataGenerationMethod] = DEFAULT_DATA_GENERATION_METHODS,
+    force_schema_version: Optional[str] = None,
     **kwargs: Any,
 ) -> BaseOpenAPISchema:
     """Load a remote resource and parse to schema instance."""
-    kwargs.setdefault("headers", {}).setdefault("User-Agent", USER_AGENT)
+    headers = kwargs.setdefault("headers", {})
+    if "user-agent" not in {header.lower() for header in headers}:
+        kwargs["headers"]["User-Agent"] = USER_AGENT
+    if not base_url and port:
+        base_url = str(URL(uri).with_port(port))
     response = requests.get(uri, **kwargs)
     try:
         response.raise_for_status()
@@ -76,6 +90,9 @@ def from_uri(
         operation_id=operation_id,
         app=app,
         validate_schema=validate_schema,
+        skip_deprecated_endpoints=skip_deprecated_endpoints,
+        data_generation_methods=data_generation_methods,
+        force_schema_version=force_schema_version,
     )
 
 
@@ -90,6 +107,9 @@ def from_file(
     *,
     app: Any = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Iterable[DataGenerationMethod] = DEFAULT_DATA_GENERATION_METHODS,
+    force_schema_version: Optional[str] = None,
     **kwargs: Any,  # needed in the runner to have compatible API across all loaders
 ) -> BaseOpenAPISchema:
     """Load a file content and parse to schema instance.
@@ -107,6 +127,9 @@ def from_file(
         operation_id=operation_id,
         app=app,
         validate_schema=validate_schema,
+        skip_deprecated_endpoints=skip_deprecated_endpoints,
+        data_generation_methods=data_generation_methods,
+        force_schema_version=force_schema_version,
     )
 
 
@@ -121,10 +144,14 @@ def from_dict(
     *,
     app: Any = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Iterable[DataGenerationMethod] = DEFAULT_DATA_GENERATION_METHODS,
+    force_schema_version: Optional[str] = None,
 ) -> BaseOpenAPISchema:
     """Get a proper abstraction for the given raw schema."""
     dispatch("before_load_schema", HookContext(), raw_schema)
-    if "swagger" in raw_schema:
+
+    def init_openapi_2() -> SwaggerV20:
         _maybe_validate_schema(raw_schema, definitions.SWAGGER_20, validate_schema)
         return SwaggerV20(
             raw_schema,
@@ -136,9 +163,11 @@ def from_dict(
             operation_id=operation_id,
             app=app,
             validate_schema=validate_schema,
+            skip_deprecated_endpoints=skip_deprecated_endpoints,
+            data_generation_methods=data_generation_methods,
         )
 
-    if "openapi" in raw_schema:
+    def init_openapi_3() -> OpenApi30:
         _maybe_validate_schema(raw_schema, definitions.OPENAPI_30, validate_schema)
         return OpenApi30(
             raw_schema,
@@ -150,7 +179,18 @@ def from_dict(
             operation_id=operation_id,
             app=app,
             validate_schema=validate_schema,
+            skip_deprecated_endpoints=skip_deprecated_endpoints,
+            data_generation_methods=data_generation_methods,
         )
+
+    if force_schema_version == "20":
+        return init_openapi_2()
+    if force_schema_version == "30":
+        return init_openapi_3()
+    if "swagger" in raw_schema:
+        return init_openapi_2()
+    if "openapi" in raw_schema:
+        return init_openapi_3()
     raise ValueError("Unsupported schema type")
 
 
@@ -169,6 +209,8 @@ def from_pytest_fixture(
     tag: Optional[Filter] = NOT_SET,
     operation_id: Optional[Filter] = NOT_SET,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Iterable[DataGenerationMethod] = DEFAULT_DATA_GENERATION_METHODS,
 ) -> LazySchema:
     """Needed for a consistent library API."""
     return LazySchema(
@@ -178,6 +220,8 @@ def from_pytest_fixture(
         tag=tag,
         operation_id=operation_id,
         validate_schema=validate_schema,
+        skip_deprecated_endpoints=skip_deprecated_endpoints,
+        data_generation_methods=data_generation_methods,
     )
 
 
@@ -190,9 +234,14 @@ def from_wsgi(
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Iterable[DataGenerationMethod] = DEFAULT_DATA_GENERATION_METHODS,
+    force_schema_version: Optional[str] = None,
     **kwargs: Any,
 ) -> BaseOpenAPISchema:
-    kwargs.setdefault("headers", {}).setdefault("User-Agent", USER_AGENT)
+    headers = kwargs.setdefault("headers", {})
+    if "user-agent" not in {header.lower() for header in headers}:
+        kwargs["headers"]["User-Agent"] = USER_AGENT
     client = Client(app, WSGIResponse)
     response = client.get(schema_path, **kwargs)
     # Raising exception to provide unified behavior
@@ -209,6 +258,9 @@ def from_wsgi(
         operation_id=operation_id,
         app=app,
         validate_schema=validate_schema,
+        skip_deprecated_endpoints=skip_deprecated_endpoints,
+        data_generation_methods=data_generation_methods,
+        force_schema_version=force_schema_version,
     )
 
 
@@ -230,6 +282,9 @@ def from_aiohttp(
     operation_id: Optional[Filter] = None,
     *,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Iterable[DataGenerationMethod] = DEFAULT_DATA_GENERATION_METHODS,
+    force_schema_version: Optional[str] = None,
     **kwargs: Any,
 ) -> BaseOpenAPISchema:
     from .extra._aiohttp import run_server  # pylint: disable=import-outside-toplevel
@@ -245,6 +300,9 @@ def from_aiohttp(
         tag=tag,
         operation_id=operation_id,
         validate_schema=validate_schema,
+        skip_deprecated_endpoints=skip_deprecated_endpoints,
+        data_generation_methods=data_generation_methods,
+        force_schema_version=force_schema_version,
         **kwargs,
     )
 
@@ -257,9 +315,14 @@ def from_asgi(
     endpoint: Optional[Filter] = None,
     tag: Optional[Filter] = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Iterable[DataGenerationMethod] = DEFAULT_DATA_GENERATION_METHODS,
+    force_schema_version: Optional[str] = None,
     **kwargs: Any,
 ) -> BaseOpenAPISchema:
-    kwargs.setdefault("headers", {}).setdefault("User-Agent", USER_AGENT)
+    headers = kwargs.setdefault("headers", {})
+    if "user-agent" not in {header.lower() for header in headers}:
+        kwargs["headers"]["User-Agent"] = USER_AGENT
     client = ASGIClient(app)
     response = client.get(schema_path, **kwargs)
     # Raising exception to provide unified behavior
@@ -275,4 +338,7 @@ def from_asgi(
         tag=tag,
         app=app,
         validate_schema=validate_schema,
+        skip_deprecated_endpoints=skip_deprecated_endpoints,
+        data_generation_methods=data_generation_methods,
+        force_schema_version=force_schema_version,
     )
