@@ -1,4 +1,3 @@
-# pylint: disable=too-many-instance-attributes
 import base64
 import datetime
 import http
@@ -11,14 +10,16 @@ from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, Iterator, List
 from urllib.parse import urljoin, urlsplit, urlunsplit
 
 import attr
+import curlify
 import requests
 import werkzeug
 from hypothesis.strategies import SearchStrategy
 from starlette.testclient import TestClient as ASGIClient
 
-from .exceptions import InvalidSchema
+from .constants import USER_AGENT, DataGenerationMethod
+from .exceptions import CheckFailed, InvalidSchema, get_grouped_exception
 from .types import Body, Cookies, FormData, Headers, PathParameters, Query
-from .utils import GenericResponse, WSGIResponse
+from .utils import GenericResponse, WSGIResponse, get_response_payload
 
 if TYPE_CHECKING:
     from .hooks import HookDispatcher
@@ -33,10 +34,18 @@ MISSING_STATEFUL_ARGUMENT_MESSAGE = (
 
 
 @attr.s(slots=True)  # pragma: no mutate
-class Case:
+class CaseSource:
+    """Data sources, used to generate a test case."""
+
+    case: "Case" = attr.ib()  # pragma: no mutate
+    response: GenericResponse = attr.ib()  # pragma: no mutate
+
+
+@attr.s(slots=True, repr=False)  # pragma: no mutate
+class Case:  # pylint: disable=too-many-public-methods
     """A single test case parameters."""
 
-    endpoint: "Endpoint" = attr.ib(repr=False)  # pragma: no mutate
+    endpoint: "Endpoint" = attr.ib()  # pragma: no mutate
     path_parameters: Optional[PathParameters] = attr.ib(default=None)  # pragma: no mutate
     headers: Optional[Headers] = attr.ib(default=None)  # pragma: no mutate
     cookies: Optional[Cookies] = attr.ib(default=None)  # pragma: no mutate
@@ -44,7 +53,21 @@ class Case:
     body: Optional[Body] = attr.ib(default=None)  # pragma: no mutate
     form_data: Optional[FormData] = attr.ib(default=None)  # pragma: no mutate
 
-    feedback: "Feedback" = attr.ib(repr=False, default=None)
+    feedback: "Feedback" = attr.ib(default=None)  # pragma: no mutate
+    source: Optional[CaseSource] = attr.ib(default=None)  # pragma: no mutate
+
+    def __repr__(self) -> str:
+        parts = [f"{self.__class__.__name__}("]
+        first = True
+        for name in ("path_parameters", "headers", "cookies", "query", "body", "form_data"):
+            value = getattr(self, name)
+            if value is not None:
+                if first:
+                    first = False
+                else:
+                    parts.append(", ")
+                parts.extend((name, "=", repr(value)))
+        return "".join(parts) + ")"
 
     @property
     def path(self) -> str:
@@ -56,7 +79,7 @@ class Case:
 
     @property
     def method(self) -> str:
-        return self.endpoint.method
+        return self.endpoint.method.upper()
 
     @property
     def base_url(self) -> Optional[str]:
@@ -66,6 +89,9 @@ class Case:
     def app(self) -> Any:
         return self.endpoint.app
 
+    def set_source(self, response: GenericResponse, case: "Case") -> None:
+        self.source = CaseSource(case=case, response=response)
+
     @property
     def formatted_path(self) -> str:
         # pylint: disable=not-a-mapping
@@ -105,30 +131,47 @@ class Case:
             if (key == "Body" and value is not None) or value not in (None, {})
         ]
 
-    def get_code_to_reproduce(self, headers: Optional[Dict[str, Any]] = None) -> str:
+    def get_code_to_reproduce(
+        self, headers: Optional[Dict[str, Any]] = None, request: Optional[requests.PreparedRequest] = None
+    ) -> str:
         """Construct a Python code to reproduce this case with `requests`."""
-        base_url = self.get_full_base_url()
-        kwargs = self.as_requests_kwargs(base_url)
+        if request is not None:
+            kwargs: Dict[str, Any] = {
+                "method": request.method,
+                "url": request.url,
+                "headers": request.headers,
+                "data": request.body,
+            }
+        else:
+            base_url = self.get_full_base_url()
+            kwargs = self.as_requests_kwargs(base_url)
         if headers:
             final_headers = kwargs["headers"] or {}
             final_headers.update(headers)
             kwargs["headers"] = final_headers
         method = kwargs["method"].lower()
 
-        def are_defaults(key: str, value: Optional[Dict]) -> bool:
-            default_value: Optional[Dict] = {"json": None}.get(key, None)
-            return value == default_value
-
         printed_kwargs = ", ".join(
             f"{key}={repr(value)}"
             for key, value in kwargs.items()
-            if key not in ("method", "url") and not are_defaults(key, value)
+            if key not in ("method", "url") and value is not None
         )
         args_repr = f"'{kwargs['url']}'"
         if printed_kwargs:
             args_repr += f", {printed_kwargs}"
         return f"requests.{method}({args_repr})"
 
+    def as_curl_command(self, headers: Optional[Dict[str, Any]] = None) -> str:
+        """Construct a curl command for a given case."""
+        base_url = self.get_full_base_url()
+        kwargs = self.as_requests_kwargs(base_url)
+        if headers:
+            final_headers = kwargs["headers"] or {}
+            final_headers.update(headers)
+            kwargs["headers"] = final_headers
+        request = requests.Request(**kwargs)
+        return curlify.to_curl(request.prepare())
+
     def _get_base_url(self, base_url: Optional[str] = None) -> str:
         if base_url is None:
             if self.base_url is not None:
@@ -140,8 +183,19 @@ class Case:
                 )
         return base_url
 
-    def as_requests_kwargs(self, base_url: Optional[str] = None) -> Dict[str, Any]:
+    def _get_headers(self, headers: Optional[Dict[str, str]] = None) -> Dict[str, Any]:
+        final_headers = self.headers.copy() if self.headers is not None else {}
+        if headers:
+            final_headers.update(headers)
+        if "user-agent" not in {header.lower() for header in final_headers}:
+            final_headers["User-Agent"] = USER_AGENT
+        return final_headers
+
+    def as_requests_kwargs(
+        self, base_url: Optional[str] = None, headers: Optional[Dict[str, str]] = None
+    ) -> Dict[str, Any]:
         """Convert the case into a dictionary acceptable by requests."""
+        final_headers = self._get_headers(headers)
         base_url = self._get_base_url(base_url)
         formatted_path = self.formatted_path.lstrip("/")  # pragma: no mutate
         url = urljoin(base_url + "/", formatted_path)
@@ -158,7 +212,7 @@ class Case:
             "method": self.method,
             "url": url,
             "cookies": self.cookies,
-            "headers": self.headers,
+            "headers": final_headers,
             "params": self.query,
             **extra,
         }
@@ -176,10 +230,7 @@ class Case:
             close_session = True
         else:
             close_session = False
-
-        data = self.as_requests_kwargs(base_url)
-        if headers is not None:
-            data["headers"] = {**(data["headers"] or {}), **headers}
+        data = self.as_requests_kwargs(base_url, headers)
         data.update(kwargs)
         response = session.request(**data)  # type: ignore
         if close_session:
@@ -195,9 +246,7 @@ class Case:
 
     def as_werkzeug_kwargs(self, headers: Optional[Dict[str, str]] = None) -> Dict[str, Any]:
         """Convert the case into a dictionary acceptable by werkzeug.Client."""
-        final_headers = self.headers.copy() if self.headers is not None else {}
-        if headers:
-            final_headers.update(headers)
+        final_headers = self._get_headers(headers)
         extra: Dict[str, Optional[Body]]
         if self.form_data:
             extra = {"data": self.form_data}
@@ -227,6 +276,8 @@ class Case:
         client = werkzeug.Client(application, WSGIResponse)
         with cookie_handler(client, self.cookies):
             response = client.open(**data, **kwargs)
+        requests_kwargs = self.as_requests_kwargs(base_url=self.get_full_base_url(), headers=headers)
+        response.request = requests.Request(**requests_kwargs).prepare()
         if self.feedback:
             self.store_response(response)
         return response
@@ -252,18 +303,38 @@ class Case:
         self,
         response: GenericResponse,
         checks: Tuple["CheckFunction", ...] = (),
+        additional_checks: Tuple["CheckFunction", ...] = (),
     ) -> None:
         from .checks import ALL_CHECKS  # pylint: disable=import-outside-toplevel
 
         checks = checks or ALL_CHECKS
+        checks += additional_checks
         errors = []
         for check in checks:
             try:
                 check(response, self)
-            except AssertionError as exc:
-                errors.append(exc.args[0])
+            except CheckFailed as exc:
+                errors.append(exc)
         if errors:
-            raise AssertionError(*errors)
+            exception_cls = get_grouped_exception(self.endpoint.verbose_name, *errors)
+            formatted_errors = "\n\n".join(f"{idx}. {error.args[0]}" for idx, error in enumerate(errors, 1))
+            code = self.get_code_to_reproduce(request=response.request)
+            payload = get_response_payload(response)
+            raise exception_cls(
+                f"\n\n{formatted_errors}\n\n----------\n\nResponse payload: `{payload}`\n\n"
+                f"Run this Python code to reproduce this response: \n\n    {code}\n"
+            )
+
+    def call_and_validate(
+        self,
+        base_url: Optional[str] = None,
+        session: Optional[requests.Session] = None,
+        headers: Optional[Dict[str, Any]] = None,
+        checks: Tuple["CheckFunction", ...] = (),
+        **kwargs: Any,
+    ) -> None:
+        response = self.call(base_url, session, headers, **kwargs)
+        self.validate_response(response, checks)
 
     def get_full_url(self) -> str:
         """Make a full URL to the current endpoint, including query parameters."""
@@ -271,7 +342,7 @@ class Case:
         kwargs = self.as_requests_kwargs(base_url)
         request = requests.Request(**kwargs)
         prepared = requests.Session().prepare_request(request)  # type: ignore
-        return prepared.url
+        return prepared.url  # type: ignore
 
     def partial_deepcopy(self) -> "Case":
         return self.__class__(
@@ -335,6 +406,7 @@ class EndpointDefinition:
     raw: Dict[str, Any] = attr.ib()  # pragma: no mutate
     resolved: Dict[str, Any] = attr.ib()  # pragma: no mutate
     scope: str = attr.ib()  # pragma: no mutate
+    parameters: List[Dict[str, Any]] = attr.ib()  # pragma: no mutate
 
 
 @attr.s(slots=True)  # pragma: no mutate
@@ -346,7 +418,7 @@ class Endpoint:
     # https://swagger.io/docs/specification/2-0/api-host-and-base-path/
     path: str = attr.ib()  # pragma: no mutate
     method: str = attr.ib()  # pragma: no mutate
-    definition: EndpointDefinition = attr.ib()  # pragma: no mutate
+    definition: EndpointDefinition = attr.ib(repr=False)  # pragma: no mutate
     schema: "BaseSchema" = attr.ib()  # pragma: no mutate
     app: Any = attr.ib(default=None)  # pragma: no mutate
     base_url: Optional[str] = attr.ib(default=None)  # pragma: no mutate
@@ -357,14 +429,25 @@ class Endpoint:
     body: Optional[Dict[str, Any]] = attr.ib(default=None)  # pragma: no mutate
     form_data: Optional[Dict[str, Any]] = attr.ib(default=None)  # pragma: no mutate
 
+    @property
+    def verbose_name(self) -> str:
+        return f"{self.method.upper()} {self.path}"
+
     @property
     def full_path(self) -> str:
         return self.schema.get_full_path(self.path)
 
+    @property
+    def links(self) -> Dict[str, Dict[str, Any]]:
+        return self.schema.get_links(self)
+
     def as_strategy(
-        self, hooks: Optional["HookDispatcher"] = None, feedback: Optional["Feedback"] = None
+        self,
+        hooks: Optional["HookDispatcher"] = None,
+        feedback: Optional["Feedback"] = None,
+        data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
     ) -> SearchStrategy:
-        return self.schema.get_case_strategy(self, hooks, feedback)
+        return self.schema.get_case_strategy(self, hooks, feedback, data_generation_method)
 
     def get_strategies_from_examples(self) -> List[SearchStrategy[Case]]:
         """Get examples from the endpoint."""
@@ -393,7 +476,7 @@ class Endpoint:
             app=self.app,  # not deepcopyable
             base_url=self.base_url,  # string, immutable
             path_parameters=deepcopy(self.path_parameters),
-            headers=deepcopy(self.path_parameters),
+            headers=deepcopy(self.headers),
             cookies=deepcopy(self.cookies),
             query=deepcopy(self.query),
             body=deepcopy(self.body),
@@ -417,6 +500,41 @@ class Endpoint:
             form_data=self.form_data,
         )
 
+    def make_case(
+        self,
+        *,
+        path_parameters: Optional[PathParameters] = None,
+        headers: Optional[Headers] = None,
+        cookies: Optional[Cookies] = None,
+        query: Optional[Query] = None,
+        body: Optional[Body] = None,
+        form_data: Optional[FormData] = None,
+    ) -> Case:
+        return Case(
+            endpoint=self,
+            path_parameters=path_parameters,
+            headers=headers,
+            cookies=cookies,
+            query=query,
+            body=body,
+            form_data=form_data,
+        )
+
+    @property
+    def operation_reference(self) -> str:
+        path = self.path.replace("~", "~0").replace("/", "~1")
+        return f"#/paths/{path}/{self.method}"
+
+    def validate_response(self, response: GenericResponse) -> None:
+        return self.schema.validate_response(self, response)
+
+    def is_response_valid(self, response: GenericResponse) -> bool:
+        try:
+            self.validate_response(response)
+            return True
+        except CheckFailed:
+            return False
+
 
 class Status(IntEnum):
     """Status of an action or multiple actions."""
@@ -528,17 +646,37 @@ class Interaction:
 
     request: Request = attr.ib()  # pragma: no mutate
     response: Response = attr.ib()  # pragma: no mutate
+    checks: List[Check] = attr.ib()  # pragma: no mutate
+    status: Status = attr.ib()  # pragma: no mutate
     recorded_at: str = attr.ib(factory=lambda: datetime.datetime.now().isoformat())  # pragma: no mutate
 
     @classmethod
-    def from_requests(cls, response: requests.Response) -> "Interaction":
-        return cls(request=Request.from_prepared_request(response.request), response=Response.from_requests(response))
+    def from_requests(cls, response: requests.Response, status: Status, checks: List[Check]) -> "Interaction":
+        return cls(
+            request=Request.from_prepared_request(response.request),
+            response=Response.from_requests(response),
+            status=status,
+            checks=checks,
+        )
 
     @classmethod
-    def from_wsgi(cls, case: Case, response: WSGIResponse, headers: Dict[str, Any], elapsed: float) -> "Interaction":
+    def from_wsgi(
+        cls,
+        case: Case,
+        response: WSGIResponse,
+        headers: Dict[str, Any],
+        elapsed: float,
+        status: Status,
+        checks: List[Check],
+    ) -> "Interaction":
         session = requests.Session()
         session.headers.update(headers)
-        return cls(request=Request.from_case(case, session), response=Response.from_wsgi(response, elapsed))
+        return cls(
+            request=Request.from_case(case, session),
+            response=Response.from_wsgi(response, elapsed),
+            status=status,
+            checks=checks,
+        )
 
 
 @attr.s(slots=True, repr=False)  # pragma: no mutate
@@ -546,6 +684,7 @@ class TestResult:
     """Result of a single test."""
 
     endpoint: Endpoint = attr.ib()  # pragma: no mutate
+    data_generation_method: DataGenerationMethod = attr.ib()  # pragma: no mutate
     checks: List[Check] = attr.ib(factory=list)  # pragma: no mutate
     errors: List[Tuple[Exception, Optional[Case]]] = attr.ib(factory=list)  # pragma: no mutate
     interactions: List[Interaction] = attr.ib(factory=list)  # pragma: no mutate
@@ -579,11 +718,19 @@ class TestResult:
     def add_error(self, exception: Exception, example: Optional[Case] = None) -> None:
         self.errors.append((exception, example))
 
-    def store_requests_response(self, response: requests.Response) -> None:
-        self.interactions.append(Interaction.from_requests(response))
+    def store_requests_response(self, response: requests.Response, status: Status, checks: List[Check]) -> None:
+        self.interactions.append(Interaction.from_requests(response, status, checks))
 
-    def store_wsgi_response(self, case: Case, response: WSGIResponse, headers: Dict[str, Any], elapsed: float) -> None:
-        self.interactions.append(Interaction.from_wsgi(case, response, headers, elapsed))
+    def store_wsgi_response(
+        self,
+        case: Case,
+        response: WSGIResponse,
+        headers: Dict[str, Any],
+        elapsed: float,
+        status: Status,
+        checks: List[Check],
+    ) -> None:
+        self.interactions.append(Interaction.from_wsgi(case, response, headers, elapsed, status, checks))
 
 
 @attr.s(slots=True, repr=False)  # pragma: no mutate
