@@ -7,7 +7,7 @@ from starlette.applications import Starlette
 from .. import fixups as _fixups
 from .. import loaders
 from ..checks import DEFAULT_CHECKS
-from ..constants import DEFAULT_STATEFUL_RECURSION_LIMIT
+from ..constants import DEFAULT_DATA_GENERATION_METHODS, DEFAULT_STATEFUL_RECURSION_LIMIT, DataGenerationMethod
 from ..models import CheckFunction
 from ..schemas import BaseSchema
 from ..stateful import Stateful
@@ -26,11 +26,13 @@ from .impl import (
 )
 
 
-def prepare(  # pylint: disable=too-many-arguments
+def prepare(
     schema_uri: Union[str, Dict[str, Any]],
     *,
     # Runtime behavior
     checks: Iterable[CheckFunction] = DEFAULT_CHECKS,
+    data_generation_methods: Tuple[DataGenerationMethod, ...] = DEFAULT_DATA_GENERATION_METHODS,
+    max_response_time: Optional[int] = None,
     targets: Iterable[Target] = DEFAULT_TARGETS,
     workers_num: int = 1,
     seed: Optional[int] = None,
@@ -46,12 +48,15 @@ def prepare(  # pylint: disable=too-many-arguments
     auth_type: Optional[str] = None,
     headers: Optional[Dict[str, str]] = None,
     request_timeout: Optional[int] = None,
+    request_tls_verify: Union[bool, str] = True,
     endpoint: Optional[Filter] = None,
     method: Optional[Filter] = None,
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     app: Optional[str] = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    force_schema_version: Optional[str] = None,
     # Hypothesis-specific configuration
     hypothesis_deadline: Optional[Union[int, NotSet]] = None,
     hypothesis_derandomize: Optional[bool] = None,
@@ -88,7 +93,11 @@ def prepare(  # pylint: disable=too-many-arguments
         operation_id=operation_id,
         app=app,
         validate_schema=validate_schema,
+        skip_deprecated_endpoints=skip_deprecated_endpoints,
+        force_schema_version=force_schema_version,
         checks=checks,
+        data_generation_methods=data_generation_methods,
+        max_response_time=max_response_time,
         targets=targets,
         hypothesis_options=hypothesis_options,
         seed=seed,
@@ -98,6 +107,7 @@ def prepare(  # pylint: disable=too-many-arguments
         auth_type=auth_type,
         headers=headers,
         request_timeout=request_timeout,
+        request_tls_verify=request_tls_verify,
         store_interactions=store_interactions,
         fixups=fixups,
         stateful=stateful,
@@ -135,7 +145,11 @@ def execute_from_schema(
     operation_id: Optional[Filter] = None,
     app: Optional[str] = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    force_schema_version: Optional[str] = None,
     checks: Iterable[CheckFunction],
+    data_generation_methods: Tuple[DataGenerationMethod, ...] = DEFAULT_DATA_GENERATION_METHODS,
+    max_response_time: Optional[int] = None,
     targets: Iterable[Target],
     workers_num: int = 1,
     hypothesis_options: Dict[str, Any],
@@ -143,6 +157,7 @@ def execute_from_schema(
     auth_type: Optional[str] = None,
     headers: Optional[Dict[str, Any]] = None,
     request_timeout: Optional[int] = None,
+    request_tls_verify: Union[bool, str] = True,
     seed: Optional[int] = None,
     exit_first: bool = False,
     store_interactions: bool = False,
@@ -170,6 +185,7 @@ def execute_from_schema(
             loader=loader,
             app=app,
             validate_schema=validate_schema,
+            skip_deprecated_endpoints=skip_deprecated_endpoints,
             auth=auth,
             auth_type=auth_type,
             headers=headers,
@@ -177,6 +193,9 @@ def execute_from_schema(
             method=method,
             tag=tag,
             operation_id=operation_id,
+            data_generation_methods=data_generation_methods,
+            force_schema_version=force_schema_version,
+            request_tls_verify=request_tls_verify,
         )
 
         runner: BaseRunner
@@ -185,6 +204,7 @@ def execute_from_schema(
                 runner = ThreadPoolRunner(
                     schema=schema,
                     checks=checks,
+                    max_response_time=max_response_time,
                     targets=targets,
                     hypothesis_settings=hypothesis_options,
                     auth=auth,
@@ -193,6 +213,7 @@ def execute_from_schema(
                     seed=seed,
                     workers_num=workers_num,
                     request_timeout=request_timeout,
+                    request_tls_verify=request_tls_verify,
                     exit_first=exit_first,
                     store_interactions=store_interactions,
                     stateful=stateful,
@@ -202,6 +223,7 @@ def execute_from_schema(
                 runner = ThreadPoolASGIRunner(
                     schema=schema,
                     checks=checks,
+                    max_response_time=max_response_time,
                     targets=targets,
                     hypothesis_settings=hypothesis_options,
                     auth=auth,
@@ -218,6 +240,7 @@ def execute_from_schema(
                 runner = ThreadPoolWSGIRunner(
                     schema=schema,
                     checks=checks,
+                    max_response_time=max_response_time,
                     targets=targets,
                     hypothesis_settings=hypothesis_options,
                     auth=auth,
@@ -236,6 +259,7 @@ def execute_from_schema(
                 runner = SingleThreadRunner(
                     schema=schema,
                     checks=checks,
+                    max_response_time=max_response_time,
                     targets=targets,
                     hypothesis_settings=hypothesis_options,
                     auth=auth,
@@ -243,6 +267,7 @@ def execute_from_schema(
                     headers=headers,
                     seed=seed,
                     request_timeout=request_timeout,
+                    request_tls_verify=request_tls_verify,
                     exit_first=exit_first,
                     store_interactions=store_interactions,
                     stateful=stateful,
@@ -252,6 +277,7 @@ def execute_from_schema(
                 runner = SingleThreadASGIRunner(
                     schema=schema,
                     checks=checks,
+                    max_response_time=max_response_time,
                     targets=targets,
                     hypothesis_settings=hypothesis_options,
                     auth=auth,
@@ -267,6 +293,7 @@ def execute_from_schema(
                 runner = SingleThreadWSGIRunner(
                     schema=schema,
                     checks=checks,
+                    max_response_time=max_response_time,
                     targets=targets,
                     hypothesis_settings=hypothesis_options,
                     auth=auth,
@@ -291,6 +318,10 @@ def load_schema(
     loader: Callable = loaders.from_uri,
     app: Any = None,
     validate_schema: bool = True,
+    skip_deprecated_endpoints: bool = False,
+    data_generation_methods: Tuple[DataGenerationMethod, ...] = DEFAULT_DATA_GENERATION_METHODS,
+    force_schema_version: Optional[str] = None,
+    request_tls_verify: Union[bool, str] = True,
     # Network request parameters
     auth: Optional[Tuple[str, str]] = None,
     auth_type: Optional[str] = None,
@@ -303,7 +334,13 @@ def load_schema(
 ) -> BaseSchema:
     """Load schema via specified loader and parameters."""
     loader_options = dict_true_values(
-        base_url=base_url, endpoint=endpoint, method=method, tag=tag, operation_id=operation_id, app=app
+        base_url=base_url,
+        endpoint=endpoint,
+        method=method,
+        tag=tag,
+        operation_id=operation_id,
+        app=app,
+        data_generation_methods=data_generation_methods,
     )
 
     if not isinstance(schema_uri, dict):
@@ -319,11 +356,19 @@ def load_schema(
 
     if loader is loaders.from_uri and loader_options.get("auth"):
         loader_options["auth"] = get_requests_auth(loader_options["auth"], loader_options.pop("auth_type", None))
+    if loader in (loaders.from_uri, loaders.from_aiohttp):
+        loader_options["verify"] = request_tls_verify
 
-    return loader(schema_uri, validate_schema=validate_schema, **loader_options)
+    return loader(
+        schema_uri,
+        validate_schema=validate_schema,
+        skip_deprecated_endpoints=skip_deprecated_endpoints,
+        force_schema_version=force_schema_version,
+        **loader_options,
+    )
 
 
-def prepare_hypothesis_options(  # pylint: disable=too-many-arguments
+def prepare_hypothesis_options(
     deadline: Optional[Union[int, NotSet]] = None,
     derandomize: Optional[bool] = None,
     max_examples: Optional[int] = None,
