@@ -1,4 +1,3 @@
-# pylint: disable=too-many-instance-attributes,too-many-arguments
 import time
 from typing import Dict, List, Optional, Union
 
@@ -40,8 +39,17 @@ class Initialized(ExecutionEvent):
         )
 
 
+class CurrentPathMixin:
+    method: str
+    path: str
+
+    @property
+    def current_endpoint(self) -> str:
+        return f"{self.method} {self.path}"
+
+
 @attr.s(slots=True)  # pragma: no mutate
-class BeforeExecution(ExecutionEvent):
+class BeforeExecution(CurrentPathMixin, ExecutionEvent):
     """Happens before each examined endpoint.
 
     It happens before a single hypothesis test, that may contain many examples inside.
@@ -49,19 +57,26 @@ class BeforeExecution(ExecutionEvent):
 
     method: str = attr.ib()  # pragma: no mutate
     path: str = attr.ib()  # pragma: no mutate
+    relative_path: str = attr.ib()  # pragma: no mutate
     recursion_level: int = attr.ib()  # pragma: no mutate
 
     @classmethod
     def from_endpoint(cls, endpoint: Endpoint, recursion_level: int) -> "BeforeExecution":
-        return cls(method=endpoint.method, path=endpoint.full_path, recursion_level=recursion_level)
+        return cls(
+            method=endpoint.method.upper(),
+            path=endpoint.full_path,
+            relative_path=endpoint.path,
+            recursion_level=recursion_level,
+        )
 
 
 @attr.s(slots=True)  # pragma: no mutate
-class AfterExecution(ExecutionEvent):
+class AfterExecution(CurrentPathMixin, ExecutionEvent):
     """Happens after each examined endpoint."""
 
     method: str = attr.ib()  # pragma: no mutate
     path: str = attr.ib()  # pragma: no mutate
+    relative_path: str = attr.ib()  # pragma: no mutate
 
     # Endpoint test status - success / failure / error
     status: Status = attr.ib()  # pragma: no mutate
@@ -76,8 +91,9 @@ class AfterExecution(ExecutionEvent):
         cls, result: TestResult, status: Status, elapsed_time: float, hypothesis_output: List[str], endpoint: Endpoint
     ) -> "AfterExecution":
         return cls(
-            method=endpoint.method,
+            method=endpoint.method.upper(),
             path=endpoint.full_path,
+            relative_path=endpoint.path,
             result=SerializedTestResult.from_test_result(result),
             status=status,
             elapsed_time=elapsed_time,
