@@ -9,10 +9,10 @@ import requests
 from _pytest.logging import LogCaptureHandler, catching_logs
 from requests.auth import HTTPDigestAuth, _basic_auth_str
 
-from ...constants import DEFAULT_DEADLINE, DEFAULT_STATEFUL_RECURSION_LIMIT, USER_AGENT
+from ...constants import DEFAULT_DEADLINE, DEFAULT_STATEFUL_RECURSION_LIMIT, USER_AGENT, DataGenerationMethod
 from ...exceptions import CheckFailed, InvalidSchema, get_grouped_exception
 from ...hooks import HookContext, get_all_by_name
-from ...models import Case, CheckFunction, Endpoint, Status, TestResult, TestResultSet
+from ...models import Case, Check, CheckFunction, Endpoint, Status, TestResult, TestResultSet
 from ...runner import events
 from ...schemas import BaseSchema
 from ...stateful import Feedback, Stateful
@@ -27,11 +27,11 @@ def get_hypothesis_settings(hypothesis_options: Dict[str, Any]) -> hypothesis.se
     return hypothesis.settings(**hypothesis_options)
 
 
-# pylint: disable=too-many-instance-attributes
 @attr.s  # pragma: no mutate
 class BaseRunner:
     schema: BaseSchema = attr.ib()  # pragma: no mutate
     checks: Iterable[CheckFunction] = attr.ib()  # pragma: no mutate
+    max_response_time: Optional[int] = attr.ib()  # pragma: no mutate
     targets: Iterable[Target] = attr.ib()  # pragma: no mutate
     hypothesis_settings: hypothesis.settings = attr.ib(converter=get_hypothesis_settings)  # pragma: no mutate
     auth: Optional[RawAuth] = attr.ib(default=None)  # pragma: no mutate
@@ -65,7 +65,7 @@ class BaseRunner:
     def _execute(self, results: TestResultSet) -> Generator[events.ExecutionEvent, None, None]:
         raise NotImplementedError
 
-    def _run_tests(  # pylint: disable=too-many-arguments
+    def _run_tests(
         self,
         maker: Callable,
         template: Callable,
@@ -77,9 +77,16 @@ class BaseRunner:
         """Run tests and recursively run additional tests."""
         if recursion_level > self.stateful_recursion_limit:
             return
-        for endpoint, test in maker(template, settings, seed):
+        for endpoint, data_generation_method, test in maker(template, settings, seed):
             feedback = Feedback(self.stateful, endpoint)
-            for event in run_test(endpoint, test, feedback=feedback, recursion_level=recursion_level, **kwargs):
+            for event in run_test(
+                endpoint,
+                test,
+                feedback=feedback,
+                recursion_level=recursion_level,
+                data_generation_method=data_generation_method,
+                **kwargs,
+            ):
                 yield event
                 if isinstance(event, events.Interrupted):
                     return
@@ -93,6 +100,7 @@ def run_test(  # pylint: disable=too-many-locals
     endpoint: Endpoint,
     test: Union[Callable, InvalidSchema],
     checks: Iterable[CheckFunction],
+    data_generation_method: DataGenerationMethod,
     targets: Iterable[Target],
     results: TestResultSet,
     headers: Optional[Dict[str, Any]],
@@ -100,8 +108,7 @@ def run_test(  # pylint: disable=too-many-locals
     **kwargs: Any,
 ) -> Generator[events.ExecutionEvent, None, None]:
     """A single test run with all error handling needed."""
-    # pylint: disable=too-many-arguments
-    result = TestResult(endpoint=endpoint, overridden_headers=headers)
+    result = TestResult(endpoint=endpoint, overridden_headers=headers, data_generation_method=data_generation_method)
     yield events.BeforeExecution.from_endpoint(endpoint=endpoint, recursion_level=recursion_level)
     hypothesis_output: List[str] = []
     test_start_time = time.monotonic()
@@ -146,6 +153,7 @@ def run_test(  # pylint: disable=too-many-locals
         result.add_error(error)
     test_elapsed_time = time.monotonic() - test_start_time
     # Fetch seed value, hypothesis generates it during test execution
+    # It may be `None` if the `derandomize` config option is set to `True`
     result.seed = getattr(test, "_hypothesis_internal_use_seed", None) or getattr(
         test, "_hypothesis_internal_use_generated_seed", None
     )
@@ -171,7 +179,15 @@ def reraise(error: AssertionError) -> InvalidSchema:
         return exc
 
 
-def run_checks(case: Case, checks: Iterable[CheckFunction], result: TestResult, response: GenericResponse) -> None:
+def run_checks(
+    case: Case,
+    checks: Iterable[CheckFunction],
+    check_results: List[Check],
+    result: TestResult,
+    response: GenericResponse,
+    elapsed_time: float,
+    max_response_time: Optional[int] = None,
+) -> None:
     errors = []
 
     for check in checks:
@@ -180,6 +196,7 @@ def run_checks(case: Case, checks: Iterable[CheckFunction], result: TestResult,
             skip_check = check(response, case)
             if not skip_check:
                 result.add_success(check_name, case)
+                check_results.append(Check(check_name, Status.success, case))
         except AssertionError as exc:
             message = str(exc)
             if not message:
@@ -187,9 +204,18 @@ def run_checks(case: Case, checks: Iterable[CheckFunction], result: TestResult,
                 exc.args = (message,)
             errors.append(exc)
             result.add_failure(check_name, case, message)
+            check_results.append(Check(check_name, Status.failure, case, message))
+
+    if max_response_time:
+        if elapsed_time > max_response_time:
+            message = f"Response time exceeded the limit of {max_response_time} ms"
+            errors.append(AssertionError(message))
+            result.add_failure("max_response_time", case, message)
+        else:
+            result.add_success("max_response_time", case)
 
     if errors:
-        raise get_grouped_exception(*errors)
+        raise get_grouped_exception(case.endpoint.verbose_name, *errors)
 
 
 def run_targets(targets: Iterable[Callable], context: TargetContext) -> None:
@@ -214,18 +240,44 @@ def network_test(
     result: TestResult,
     session: requests.Session,
     request_timeout: Optional[int],
+    request_tls_verify: bool,
     store_interactions: bool,
     headers: Optional[Dict[str, Any]],
     feedback: Feedback,
+    max_response_time: Optional[int],
 ) -> None:
     """A single test body will be executed against the target."""
-    # pylint: disable=too-many-arguments
     headers = headers or {}
-    headers.setdefault("User-Agent", USER_AGENT)
+    if "user-agent" not in {header.lower() for header in headers}:
+        headers["User-Agent"] = USER_AGENT
     timeout = prepare_timeout(request_timeout)
-    response = _network_test(case, checks, targets, result, session, timeout, store_interactions, headers, feedback)
+    response = _network_test(
+        case,
+        checks,
+        targets,
+        result,
+        session,
+        timeout,
+        store_interactions,
+        headers,
+        feedback,
+        request_tls_verify,
+        max_response_time,
+    )
     add_cases(
-        case, response, _network_test, checks, targets, result, session, timeout, store_interactions, headers, feedback
+        case,
+        response,
+        _network_test,
+        checks,
+        targets,
+        result,
+        session,
+        timeout,
+        store_interactions,
+        headers,
+        feedback,
+        request_tls_verify,
+        max_response_time,
     )
 
 
@@ -239,14 +291,22 @@ def _network_test(
     store_interactions: bool,
     headers: Optional[Dict[str, Any]],
     feedback: Feedback,
+    request_tls_verify: bool,
+    max_response_time: Optional[int],
 ) -> requests.Response:
-    # pylint: disable=too-many-arguments
-    response = case.call(session=session, headers=headers, timeout=timeout)
+    response = case.call(session=session, headers=headers, timeout=timeout, verify=request_tls_verify)
     context = TargetContext(case=case, response=response, response_time=response.elapsed.total_seconds())
     run_targets(targets, context)
-    if store_interactions:
-        result.store_requests_response(response)
-    run_checks(case, checks, result, response)
+    status = Status.success
+    check_results: List[Check] = []
+    try:
+        run_checks(case, checks, check_results, result, response, context.response_time * 1000, max_response_time)
+    except CheckFailed:
+        status = Status.failure
+        raise
+    finally:
+        if store_interactions:
+            result.store_requests_response(response, status, check_results)
     feedback.add_test_case(case, response)
     return response
 
@@ -277,11 +337,13 @@ def wsgi_test(
     headers: Optional[Dict[str, Any]],
     store_interactions: bool,
     feedback: Feedback,
+    max_response_time: Optional[int],
 ) -> None:
-    # pylint: disable=too-many-arguments
     headers = _prepare_wsgi_headers(headers, auth, auth_type)
-    response = _wsgi_test(case, checks, targets, result, headers, store_interactions, feedback)
-    add_cases(case, response, _wsgi_test, checks, targets, result, headers, store_interactions, feedback)
+    response = _wsgi_test(case, checks, targets, result, headers, store_interactions, feedback, max_response_time)
+    add_cases(
+        case, response, _wsgi_test, checks, targets, result, headers, store_interactions, feedback, max_response_time
+    )
 
 
 def _wsgi_test(
@@ -292,18 +354,25 @@ def _wsgi_test(
     headers: Dict[str, Any],
     store_interactions: bool,
     feedback: Feedback,
+    max_response_time: Optional[int],
 ) -> WSGIResponse:
-    # pylint: disable=too-many-arguments
     with catching_logs(LogCaptureHandler(), level=logging.DEBUG) as recorded:
         start = time.monotonic()
         response = case.call_wsgi(headers=headers)
         elapsed = time.monotonic() - start
     context = TargetContext(case=case, response=response, response_time=elapsed)
     run_targets(targets, context)
-    if store_interactions:
-        result.store_wsgi_response(case, response, headers, elapsed)
     result.logs.extend(recorded.records)
-    run_checks(case, checks, result, response)
+    status = Status.success
+    check_results: List[Check] = []
+    try:
+        run_checks(case, checks, check_results, result, response, context.response_time * 1000, max_response_time)
+    except CheckFailed:
+        status = Status.failure
+        raise
+    finally:
+        if store_interactions:
+            result.store_wsgi_response(case, response, headers, elapsed, status, check_results)
     feedback.add_test_case(case, response)
     return response
 
@@ -312,7 +381,8 @@ def _prepare_wsgi_headers(
     headers: Optional[Dict[str, Any]], auth: Optional[RawAuth], auth_type: Optional[str]
 ) -> Dict[str, Any]:
     headers = headers or {}
-    headers.setdefault("User-Agent", USER_AGENT)
+    if "user-agent" not in {header.lower() for header in headers}:
+        headers["User-Agent"] = USER_AGENT
     wsgi_auth = get_wsgi_auth(auth, auth_type)
     if wsgi_auth:
         headers["Authorization"] = wsgi_auth
@@ -335,13 +405,15 @@ def asgi_test(
     store_interactions: bool,
     headers: Optional[Dict[str, Any]],
     feedback: Feedback,
+    max_response_time: Optional[int],
 ) -> None:
     """A single test body will be executed against the target."""
-    # pylint: disable=too-many-arguments
     headers = headers or {}
 
-    response = _asgi_test(case, checks, targets, result, store_interactions, headers, feedback)
-    add_cases(case, response, _asgi_test, checks, targets, result, store_interactions, headers, feedback)
+    response = _asgi_test(case, checks, targets, result, store_interactions, headers, feedback, max_response_time)
+    add_cases(
+        case, response, _asgi_test, checks, targets, result, store_interactions, headers, feedback, max_response_time
+    )
 
 
 def _asgi_test(
@@ -352,13 +424,20 @@ def _asgi_test(
     store_interactions: bool,
     headers: Optional[Dict[str, Any]],
     feedback: Feedback,
+    max_response_time: Optional[int],
 ) -> requests.Response:
-    # pylint: disable=too-many-arguments
     response = case.call_asgi(headers=headers)
     context = TargetContext(case=case, response=response, response_time=response.elapsed.total_seconds())
     run_targets(targets, context)
-    if store_interactions:
-        result.store_requests_response(response)
-    run_checks(case, checks, result, response)
+    status = Status.success
+    check_results: List[Check] = []
+    try:
+        run_checks(case, checks, check_results, result, response, context.response_time * 1000, max_response_time)
+    except CheckFailed:
+        status = Status.failure
+        raise
+    finally:
+        if store_interactions:
+            result.store_requests_response(response, status, check_results)
     feedback.add_test_case(case, response)
     return response
