@@ -1,5 +1,5 @@
 # weird mypy bug with imports
-from typing import Any, Dict, Generator  # pylint: disable=unused-import
+from typing import Any, Dict, Generator, Union  # pylint: disable=unused-import
 
 import attr
 
@@ -13,6 +13,8 @@ from .core import BaseRunner, asgi_test, get_session, network_test, wsgi_test
 class SingleThreadRunner(BaseRunner):
     """Fast runner that runs tests sequentially in the main thread."""
 
+    request_tls_verify: Union[bool, str] = attr.ib(default=True)  # pragma: no mutate
+
     def _execute(self, results: TestResultSet) -> Generator[events.ExecutionEvent, None, None]:
         auth = get_requests_auth(self.auth, self.auth_type)
         with get_session(auth) as session:
@@ -22,11 +24,13 @@ class SingleThreadRunner(BaseRunner):
                 self.hypothesis_settings,
                 self.seed,
                 checks=self.checks,
+                max_response_time=self.max_response_time,
                 targets=self.targets,
                 results=results,
                 session=session,
                 headers=self.headers,
                 request_timeout=self.request_timeout,
+                request_tls_verify=self.request_tls_verify,
                 store_interactions=self.store_interactions,
             )
 
@@ -40,6 +44,7 @@ class SingleThreadWSGIRunner(SingleThreadRunner):
             self.hypothesis_settings,
             self.seed,
             checks=self.checks,
+            max_response_time=self.max_response_time,
             targets=self.targets,
             results=results,
             auth=self.auth,
@@ -58,6 +63,7 @@ class SingleThreadASGIRunner(SingleThreadRunner):
             self.hypothesis_settings,
             self.seed,
             checks=self.checks,
+            max_response_time=self.max_response_time,
             targets=self.targets,
             results=results,
             headers=self.headers,
