@@ -2,7 +2,7 @@ import ctypes
 import threading
 import time
 from queue import Queue
-from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, cast
+from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Union, cast
 
 import attr
 import hypothesis
@@ -30,23 +30,29 @@ def _run_task(
     stateful_recursion_limit: int,
     **kwargs: Any,
 ) -> None:
-    # pylint: disable=too-many-arguments
-
     def _run_tests(maker: Callable, recursion_level: int = 0) -> None:
         if recursion_level > stateful_recursion_limit:
             return
-        for _endpoint, test in maker(test_template, settings, seed):
+        for _endpoint, data_generation_method, test in maker(test_template, settings, seed):
             feedback = Feedback(stateful, _endpoint)
             for event in run_test(
-                _endpoint, test, checks, targets, results, recursion_level=recursion_level, feedback=feedback, **kwargs
+                _endpoint,
+                test,
+                checks,
+                data_generation_method,
+                targets,
+                results,
+                recursion_level=recursion_level,
+                feedback=feedback,
+                **kwargs,
             ):
                 events_queue.put(event)
             _run_tests(feedback.get_stateful_tests, recursion_level + 1)
 
     with capture_hypothesis_output():
         while not tasks_queue.empty():
-            endpoint = tasks_queue.get()
-            items = (endpoint, make_test_or_exception(endpoint, test_template, settings, seed))
+            endpoint, data_generation_method = tasks_queue.get()
+            items = (endpoint, data_generation_method, make_test_or_exception(endpoint, test_template, settings, seed))
             # This lambda ignores the input arguments to support the same interface for `feedback.get_stateful_tests`
             _run_tests(lambda *_: (items,))
 
@@ -70,7 +76,6 @@ def thread_task(
 
     Pretty similar to the default one-thread flow, but includes communication with the main thread via the events queue.
     """
-    # pylint: disable=too-many-arguments
     prepared_auth = get_requests_auth(auth, auth_type)
     with get_session(prepared_auth) as session:
         _run_task(
@@ -102,7 +107,6 @@ def wsgi_thread_task(
     stateful_recursion_limit: int,
     kwargs: Any,
 ) -> None:
-    # pylint: disable=too-many-arguments
     _run_task(
         wsgi_test,
         tasks_queue,
@@ -131,7 +135,6 @@ def asgi_thread_task(
     stateful_recursion_limit: int,
     kwargs: Any,
 ) -> None:
-    # pylint: disable=too-many-arguments
     _run_task(
         asgi_test,
         tasks_queue,
@@ -158,6 +161,7 @@ class ThreadPoolRunner(BaseRunner):
     """Spread different tests among multiple worker threads."""
 
     workers_num: int = attr.ib(default=2)  # pragma: no mutate
+    request_tls_verify: Union[bool, str] = attr.ib(default=True)  # pragma: no mutate
 
     def _execute(self, results: TestResultSet) -> Generator[events.ExecutionEvent, None, None]:
         """All events come from a queue where different workers push their events."""
@@ -198,7 +202,13 @@ class ThreadPoolRunner(BaseRunner):
     def _get_tasks_queue(self) -> Queue:
         """All endpoints are distributed among all workers via a queue."""
         tasks_queue: Queue = Queue()
-        tasks_queue.queue.extend(self.schema.get_all_endpoints())
+        tasks_queue.queue.extend(
+            [
+                (endpoint, data_generation_method)
+                for endpoint in self.schema.get_all_endpoints()
+                for data_generation_method in self.schema.data_generation_methods
+            ]
+        )
         return tasks_queue
 
     def _init_workers(self, tasks_queue: Queue, events_queue: Queue, results: TestResultSet) -> List[threading.Thread]:
@@ -230,7 +240,12 @@ class ThreadPoolRunner(BaseRunner):
             "results": results,
             "stateful": self.stateful,
             "stateful_recursion_limit": self.stateful_recursion_limit,
-            "kwargs": {"request_timeout": self.request_timeout, "store_interactions": self.store_interactions},
+            "kwargs": {
+                "request_timeout": self.request_timeout,
+                "request_tls_verify": self.request_tls_verify,
+                "store_interactions": self.store_interactions,
+                "max_response_time": self.max_response_time,
+            },
         }
 
 
@@ -254,6 +269,7 @@ class ThreadPoolWSGIRunner(ThreadPoolRunner):
                 "auth_type": self.auth_type,
                 "headers": self.headers,
                 "store_interactions": self.store_interactions,
+                "max_response_time": self.max_response_time,
             },
         }
 
@@ -274,7 +290,10 @@ class ThreadPoolASGIRunner(ThreadPoolRunner):
             "results": results,
             "stateful": self.stateful,
             "stateful_recursion_limit": self.stateful_recursion_limit,
-            "kwargs": {"store_interactions": self.store_interactions},
+            "kwargs": {
+                "store_interactions": self.store_interactions,
+                "max_response_time": self.max_response_time,
+            },
         }
 
 
