@@ -2,13 +2,12 @@
 
 They all consist of primitive types and don't have references to schemas, app, etc.
 """
-# pylint: disable=too-many-instance-attributes
 import logging
 from typing import Any, Dict, List, Optional
 
 import attr
 
-from ..models import Case, Check, Interaction, Status, TestResult
+from ..models import Case, Check, Interaction, Request, Response, Status, TestResult
 from ..utils import format_exception
 
 
@@ -59,6 +58,25 @@ class SerializedError:
         )
 
 
+@attr.s(slots=True)  # pragma: no mutate
+class SerializedInteraction:
+    request: Request = attr.ib()  # pragma: no mutate
+    response: Response = attr.ib()  # pragma: no mutate
+    checks: List[SerializedCheck] = attr.ib()  # pragma: no mutate
+    status: Status = attr.ib()  # pragma: no mutate
+    recorded_at: str = attr.ib()  # pragma: no mutate
+
+    @classmethod
+    def from_interaction(cls, interaction: Interaction, headers: Optional[Dict[str, Any]]) -> "SerializedInteraction":
+        return cls(
+            request=interaction.request,
+            response=interaction.response,
+            checks=[SerializedCheck.from_check(check, headers) for check in interaction.checks],
+            status=interaction.status,
+            recorded_at=interaction.recorded_at,
+        )
+
+
 @attr.s(slots=True)  # pragma: no mutate
 class SerializedTestResult:
     method: str = attr.ib()  # pragma: no mutate
@@ -68,24 +86,29 @@ class SerializedTestResult:
     has_logs: bool = attr.ib()  # pragma: no mutate
     is_errored: bool = attr.ib()  # pragma: no mutate
     seed: Optional[int] = attr.ib()  # pragma: no mutate
+    data_generation_method: str = attr.ib()  # pragma: no mutate
     checks: List[SerializedCheck] = attr.ib()  # pragma: no mutate
     logs: List[str] = attr.ib()  # pragma: no mutate
     errors: List[SerializedError] = attr.ib()  # pragma: no mutate
-    interactions: List[Interaction] = attr.ib()  # pragma: no mutate
+    interactions: List[SerializedInteraction] = attr.ib()  # pragma: no mutate
 
     @classmethod
     def from_test_result(cls, result: TestResult) -> "SerializedTestResult":
         formatter = logging.Formatter("[%(asctime)s] %(levelname)s in %(module)s: %(message)s")
         return SerializedTestResult(
-            method=result.endpoint.method,
+            method=result.endpoint.method.upper(),
             path=result.endpoint.full_path,
             has_failures=result.has_failures,
             has_errors=result.has_errors,
             has_logs=result.has_logs,
             is_errored=result.is_errored,
             seed=result.seed,
+            data_generation_method=result.data_generation_method.as_short_name(),
             checks=[SerializedCheck.from_check(check, headers=result.overridden_headers) for check in result.checks],
             logs=[formatter.format(record) for record in result.logs],
             errors=[SerializedError.from_error(*error, headers=result.overridden_headers) for error in result.errors],
-            interactions=result.interactions,
+            interactions=[
+                SerializedInteraction.from_interaction(interaction, headers=result.overridden_headers)
+                for interaction in result.interactions
+            ],
         )
