@@ -1,4 +1,3 @@
-# pylint: disable=too-many-instance-attributes
 """Schema objects provide a convenient interface to raw schemas.
 
 Their responsibilities:
@@ -7,21 +6,23 @@ Their responsibilities:
 
 They give only static definitions of endpoints.
 """
+import warnings
 from collections.abc import Mapping
-from typing import Any, Callable, Dict, Generator, Iterator, List, Optional, Sequence, Tuple, Union
+from typing import Any, Callable, Dict, Generator, Iterable, Iterator, List, Optional, Sequence, Tuple, Type, Union
 from urllib.parse import urljoin, urlsplit, urlunsplit
 
 import attr
 import hypothesis
 from hypothesis.strategies import SearchStrategy
+from hypothesis.utils.conventions import InferType
 from requests.structures import CaseInsensitiveDict
 
 from ._hypothesis import make_test_or_exception
-from .constants import DEFAULT_STATEFUL_RECURSION_LIMIT
+from .constants import DEFAULT_DATA_GENERATION_METHODS, DEFAULT_STATEFUL_RECURSION_LIMIT, DataGenerationMethod
 from .exceptions import InvalidSchema
 from .hooks import HookContext, HookDispatcher, HookScope, dispatch
 from .models import Case, Endpoint
-from .stateful import Feedback, Stateful, StatefulTest
+from .stateful import APIStateMachine, Feedback, Stateful, StatefulTest
 from .types import Filter, FormData, GenericTest, NotSet
 from .utils import NOT_SET, GenericResponse
 
@@ -39,8 +40,12 @@ class BaseSchema(Mapping):
     hooks: HookDispatcher = attr.ib(factory=lambda: HookDispatcher(scope=HookScope.SCHEMA))  # pragma: no mutate
     test_function: Optional[GenericTest] = attr.ib(default=None)  # pragma: no mutate
     validate_schema: bool = attr.ib(default=True)  # pragma: no mutate
+    skip_deprecated_endpoints: bool = attr.ib(default=False)  # pragma: no mutate
     stateful: Optional[Stateful] = attr.ib(default=None)  # pragma: no mutate
     stateful_recursion_limit: int = attr.ib(default=DEFAULT_STATEFUL_RECURSION_LIMIT)  # pragma: no mutate
+    data_generation_methods: Iterable[DataGenerationMethod] = attr.ib(
+        default=DEFAULT_DATA_GENERATION_METHODS
+    )  # pragma: no mutate
 
     def __iter__(self) -> Iterator[str]:
         return iter(self.endpoints)
@@ -115,23 +120,29 @@ class BaseSchema(Mapping):
         raise NotImplementedError
 
     def get_all_tests(
-        self, func: Callable, settings: Optional[hypothesis.settings] = None, seed: Optional[int] = None
-    ) -> Generator[Tuple[Endpoint, Union[Callable, InvalidSchema]], None, None]:
+        self,
+        func: Callable,
+        settings: Optional[hypothesis.settings] = None,
+        seed: Optional[int] = None,
+    ) -> Generator[Tuple[Endpoint, DataGenerationMethod, Union[Callable, InvalidSchema]], None, None]:
         """Generate all endpoints and Hypothesis tests for them."""
         test: Union[Callable, InvalidSchema]
         for endpoint in self.get_all_endpoints():
-            test = make_test_or_exception(endpoint, func, settings, seed)
-            yield endpoint, test
+            for data_generation_method in self.data_generation_methods:
+                test = make_test_or_exception(endpoint, func, settings, seed, data_generation_method)
+                yield endpoint, data_generation_method, test
 
-    def parametrize(  # pylint: disable=too-many-arguments
+    def parametrize(
         self,
         method: Optional[Filter] = NOT_SET,
         endpoint: Optional[Filter] = NOT_SET,
         tag: Optional[Filter] = NOT_SET,
         operation_id: Optional[Filter] = NOT_SET,
         validate_schema: Union[bool, NotSet] = NOT_SET,
+        skip_deprecated_endpoints: Union[bool, NotSet] = NOT_SET,
         stateful: Optional[Union[Stateful, NotSet]] = NOT_SET,
         stateful_recursion_limit: Union[int, NotSet] = NOT_SET,
+        data_generation_methods: Union[Iterable[DataGenerationMethod], NotSet] = NOT_SET,
     ) -> Callable:
         """Mark a test function as a parametrized one."""
 
@@ -144,14 +155,26 @@ class BaseSchema(Mapping):
                 tag=tag,
                 operation_id=operation_id,
                 validate_schema=validate_schema,
+                skip_deprecated_endpoints=skip_deprecated_endpoints,
                 stateful=stateful,
                 stateful_recursion_limit=stateful_recursion_limit,
+                data_generation_methods=data_generation_methods,
             )
             return func
 
         return wrapper
 
-    def clone(  # pylint: disable=too-many-arguments
+    def given(self, *args: Union[SearchStrategy, InferType], **kwargs: Union[SearchStrategy, InferType]) -> Callable:
+        """Proxy Hypothesis strategies to ``hypothesis.given``."""
+
+        def wrapper(func: GenericTest) -> GenericTest:
+            func._schemathesis_given_args = args  # type: ignore
+            func._schemathesis_given_kwargs = kwargs  # type: ignore
+            return func
+
+        return wrapper
+
+    def clone(
         self,
         *,
         test_function: Optional[GenericTest] = None,
@@ -161,9 +184,17 @@ class BaseSchema(Mapping):
         operation_id: Optional[Filter] = NOT_SET,
         hooks: Union[HookDispatcher, NotSet] = NOT_SET,
         validate_schema: Union[bool, NotSet] = NOT_SET,
+        skip_deprecated_endpoints: Union[bool, NotSet] = NOT_SET,
         stateful: Optional[Union[Stateful, NotSet]] = NOT_SET,
         stateful_recursion_limit: Union[int, NotSet] = NOT_SET,
+        data_generation_methods: Union[Iterable[DataGenerationMethod], NotSet] = NOT_SET,
     ) -> "BaseSchema":
+        if stateful is not NOT_SET or stateful_recursion_limit is not NOT_SET:
+            warnings.warn(
+                "`stateful` and `stateful_recursion_limit` arguments are deprecated. Use `as_state_machine` instead.",
+                DeprecationWarning,
+            )
+
         if method is NOT_SET:
             method = self.method
         if endpoint is NOT_SET:
@@ -174,12 +205,16 @@ class BaseSchema(Mapping):
             operation_id = self.operation_id
         if validate_schema is NOT_SET:
             validate_schema = self.validate_schema
+        if skip_deprecated_endpoints is NOT_SET:
+            skip_deprecated_endpoints = self.skip_deprecated_endpoints
         if hooks is NOT_SET:
             hooks = self.hooks
         if stateful is NOT_SET:
             stateful = self.stateful
         if stateful_recursion_limit is NOT_SET:
             stateful_recursion_limit = self.stateful_recursion_limit
+        if data_generation_methods is NOT_SET:
+            data_generation_methods = self.data_generation_methods
 
         return self.__class__(
             self.raw_schema,
@@ -193,8 +228,10 @@ class BaseSchema(Mapping):
             hooks=hooks,  # type: ignore
             test_function=test_function,
             validate_schema=validate_schema,  # type: ignore
+            skip_deprecated_endpoints=skip_deprecated_endpoints,  # type: ignore
             stateful=stateful,  # type: ignore
             stateful_recursion_limit=stateful_recursion_limit,  # type: ignore
+            data_generation_methods=data_generation_methods,  # type: ignore
         )
 
     def get_local_hook_dispatcher(self) -> Optional[HookDispatcher]:
@@ -226,10 +263,23 @@ class BaseSchema(Mapping):
         raise NotImplementedError
 
     def get_case_strategy(
-        self, endpoint: Endpoint, hooks: Optional[HookDispatcher] = None, feedback: Optional[Feedback] = None
+        self,
+        endpoint: Endpoint,
+        hooks: Optional[HookDispatcher] = None,
+        feedback: Optional[Feedback] = None,
+        data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
     ) -> SearchStrategy:
         raise NotImplementedError
 
+    def as_state_machine(self) -> Type[APIStateMachine]:
+        raise NotImplementedError
+
+    def get_links(self, endpoint: Endpoint) -> Dict[str, Dict[str, Any]]:
+        raise NotImplementedError
+
+    def validate_response(self, endpoint: Endpoint, response: GenericResponse) -> None:
+        raise NotImplementedError
+
 
 def endpoints_to_dict(endpoints: Generator[Endpoint, None, None]) -> Dict[str, CaseInsensitiveDict]:
     output: Dict[str, CaseInsensitiveDict] = {}
