@@ -8,6 +8,7 @@ from hypothesis import strategies as st
 from hypothesis.strategies import SearchStrategy
 from hypothesis_graphql import strategies as gql_st
 
+from ... import DataGenerationMethod
 from ...checks import not_a_server_error
 from ...hooks import HookDispatcher
 from ...models import Case, CheckFunction, Endpoint
@@ -18,14 +19,15 @@ from ...utils import GenericResponse
 
 @attr.s()  # pragma: no mutate
 class GraphQLCase(Case):
-    def as_requests_kwargs(self, base_url: Optional[str] = None) -> Dict[str, Any]:
+    def as_requests_kwargs(
+        self, base_url: Optional[str] = None, headers: Optional[Dict[str, str]] = None
+    ) -> Dict[str, Any]:
+        final_headers = self._get_headers(headers)
         base_url = self._get_base_url(base_url)
-        return {"method": self.method, "url": base_url, "json": {"query": self.body}}
+        return {"method": self.method, "url": base_url, "json": {"query": self.body}, "headers": final_headers}
 
     def as_werkzeug_kwargs(self, headers: Optional[Dict[str, str]] = None) -> Dict[str, Any]:
-        final_headers = self.headers.copy() if self.headers is not None else {}
-        if headers:
-            final_headers.update(headers)
+        final_headers = self._get_headers(headers)
         return {
             "method": self.method,
             "path": self.endpoint.schema.get_full_path(self.formatted_path),
@@ -37,8 +39,11 @@ class GraphQLCase(Case):
     def validate_response(
         self,
         response: GenericResponse,
-        checks: Tuple[CheckFunction, ...] = (not_a_server_error,),
+        checks: Tuple[CheckFunction, ...] = (),
+        additional_checks: Tuple[CheckFunction, ...] = (),
     ) -> None:
+        checks = checks or (not_a_server_error,)
+        checks += additional_checks
         return super().validate_response(response, checks)
 
 
@@ -66,7 +71,11 @@ class GraphQLSchema(BaseSchema):
         )
 
     def get_case_strategy(
-        self, endpoint: Endpoint, hooks: Optional[HookDispatcher] = None, feedback: Optional[Feedback] = None
+        self,
+        endpoint: Endpoint,
+        hooks: Optional[HookDispatcher] = None,
+        feedback: Optional[Feedback] = None,
+        data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
     ) -> SearchStrategy:
         constructor = partial(GraphQLCase, endpoint=endpoint)
         schema = graphql.build_client_schema(self.raw_schema)
