@@ -9,6 +9,7 @@ from hypothesis_jsonschema import from_schema
 from requests.auth import _basic_auth_str
 
 from ... import utils
+from ...constants import DataGenerationMethod
 from ...exceptions import InvalidSchema
 from ...hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher
 from ...models import Case, Endpoint
@@ -16,7 +17,7 @@ from ...stateful import Feedback
 
 PARAMETERS = frozenset(("path_parameters", "headers", "cookies", "query", "body", "form_data"))
 SLASH = "/"
-STRING_FORMATS = {}
+STRING_FORMATS: Dict[str, st.SearchStrategy] = {}
 
 
 def register_string_format(name: str, strategy: st.SearchStrategy) -> None:
@@ -46,8 +47,6 @@ def init_default_strategies() -> None:
 def is_valid_header(headers: Dict[str, Any]) -> bool:
     """Verify if the generated headers are valid."""
     for name, value in headers.items():
-        if not isinstance(value, str):
-            return False
         if not utils.is_latin_1_encodable(value):
             return False
         if utils.has_invalid_characters(name, value):
@@ -71,7 +70,10 @@ def is_valid_query(query: Dict[str, Any]) -> bool:
 
 
 def get_case_strategy(
-    endpoint: Endpoint, hooks: Optional[HookDispatcher] = None, feedback: Optional[Feedback] = None
+    endpoint: Endpoint,
+    hooks: Optional[HookDispatcher] = None,
+    feedback: Optional[Feedback] = None,
+    data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
 ) -> st.SearchStrategy:
     """Create a strategy for a complete test case.
 
@@ -83,7 +85,9 @@ def get_case_strategy(
         value = getattr(endpoint, parameter)
         if value is not None:
             location = {"headers": "header", "cookies": "cookie", "path_parameters": "path"}.get(parameter, parameter)
-            strategies[parameter] = prepare_strategy(parameter, value, endpoint.get_hypothesis_conversions(location))
+            strategies[parameter] = prepare_strategy(
+                parameter, value, endpoint.get_hypothesis_conversions(location), data_generation_method
+            )
         else:
             static_kwargs[parameter] = None
     return _get_case_strategy(endpoint, static_kwargs, strategies, hooks)
@@ -93,10 +97,6 @@ def to_bytes(value: Union[str, bytes, int, bool, float]) -> bytes:
     return str(value).encode(errors="ignore")
 
 
-def is_valid_form_data(form_data: Any) -> bool:
-    return isinstance(form_data, dict)
-
-
 def prepare_form_data(form_data: Dict[str, Any]) -> Dict[str, Any]:
     for name, value in form_data.items():
         if isinstance(value, list):
@@ -106,9 +106,35 @@ def prepare_form_data(form_data: Dict[str, Any]) -> Dict[str, Any]:
     return form_data
 
 
-def prepare_strategy(parameter: str, value: Dict[str, Any], map_func: Optional[Callable]) -> st.SearchStrategy:
+def prepare_headers_schema(value: Dict[str, Any]) -> Dict[str, Any]:
+    """Improve schemas for headers.
+
+    Headers are strings, but it is not always explicitly defined in the schema. By preparing them properly we
+    can achieve significant performance improvements for such cases.
+    For reference (my machine) - running a single test with 100 examples with the resulting strategy:
+      - without: 4.37 s
+      - with: 294 ms
+
+    It also reduces the number of cases when the "filter_too_much" health check fails during testing.
+    """
+    for schema in value.get("properties", {}).values():
+        schema.setdefault("type", "string")
+    return value
+
+
+def prepare_strategy(
+    parameter: str,
+    value: Dict[str, Any],
+    map_func: Optional[Callable],
+    data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
+) -> st.SearchStrategy:
     """Create a strategy for a schema and add location-specific filters & maps."""
-    strategy = from_schema(value, custom_formats=STRING_FORMATS)
+    if parameter in ("headers", "cookies"):
+        value = prepare_headers_schema(value)
+    if parameter == "form_data":
+        value.setdefault("type", "object")
+    to_strategy = {DataGenerationMethod.positive: make_positive_strategy}[data_generation_method]
+    strategy = to_strategy(value)
     if map_func is not None:
         strategy = strategy.map(map_func)
     if parameter == "path_parameters":
@@ -118,10 +144,14 @@ def prepare_strategy(parameter: str, value: Dict[str, Any], map_func: Optional[C
     elif parameter == "query":
         strategy = strategy.filter(is_valid_query)  # type: ignore
     elif parameter == "form_data":
-        strategy = strategy.filter(is_valid_form_data).map(prepare_form_data)  # type: ignore
+        strategy = strategy.map(prepare_form_data)  # type: ignore
     return strategy
 
 
+def make_positive_strategy(schema: Dict[str, Any]) -> st.SearchStrategy:
+    return from_schema(schema, custom_formats=STRING_FORMATS)
+
+
 def filter_path_parameters(parameters: Dict[str, Any]) -> bool:
     """Single "." chars and empty strings "" are excluded from path by urllib3.
 
@@ -153,7 +183,7 @@ def _get_case_strategy(
     hook_dispatcher: Optional[HookDispatcher] = None,
 ) -> st.SearchStrategy[Case]:
     static_parameters: Dict[str, Any] = {"endpoint": endpoint, **extra_static_parameters}
-    if endpoint.schema.validate_schema and endpoint.method == "GET":
+    if endpoint.schema.validate_schema and endpoint.method.upper() == "GET":
         if endpoint.body is not None:
             raise InvalidSchema("Body parameters are defined for GET request.")
         static_parameters["body"] = None
