@@ -1,26 +1,44 @@
 # pylint: disable=too-many-ancestors
 import itertools
+from collections import defaultdict
+from contextlib import ExitStack, contextmanager
 from copy import deepcopy
-from typing import Any, Callable, Dict, Generator, Iterator, List, Optional, Sequence, Tuple
+from difflib import get_close_matches
+from json import JSONDecodeError
+from typing import Any, Callable, Dict, Generator, List, Optional, Sequence, Tuple, Type, Union
 from urllib.parse import urlsplit
 
 import jsonschema
+import requests
 from hypothesis.strategies import SearchStrategy
 
-from ...exceptions import InvalidSchema
+from ...constants import DataGenerationMethod
+from ...exceptions import (
+    InvalidSchema,
+    get_missing_content_type_error,
+    get_response_parsing_error,
+    get_schema_validation_error,
+)
 from ...hooks import HookContext, HookDispatcher
 from ...models import Case, Endpoint, EndpointDefinition, empty_object
 from ...schemas import BaseSchema
-from ...stateful import Feedback, Stateful, StatefulTest
+from ...stateful import APIStateMachine, Feedback, Stateful, StatefulTest
 from ...types import FormData
-from ...utils import GenericResponse
+from ...utils import GenericResponse, get_response_payload
 from . import links, serialization
 from ._hypothesis import get_case_strategy
 from .converter import to_json_schema_recursive
 from .examples import get_strategies_from_examples
-from .filters import should_skip_by_operation_id, should_skip_by_tag, should_skip_endpoint, should_skip_method
+from .filters import (
+    should_skip_by_operation_id,
+    should_skip_by_tag,
+    should_skip_deprecated,
+    should_skip_endpoint,
+    should_skip_method,
+)
 from .references import ConvertingResolver
 from .security import BaseSecurityProcessor, OpenAPISecurityProcessor, SwaggerSecurityProcessor
+from .stateful import create_state_machine
 
 
 class BaseOpenAPISchema(BaseSchema):
@@ -62,14 +80,17 @@ class BaseOpenAPISchema(BaseSchema):
                     if (
                         method not in self.operations
                         or should_skip_method(method, self.method)
+                        or should_skip_deprecated(
+                            resolved_definition.get("deprecated", False), self.skip_deprecated_endpoints
+                        )
                         or should_skip_by_tag(resolved_definition.get("tags"), self.tag)
                         or should_skip_by_operation_id(resolved_definition.get("operationId"), self.operation_id)
                     ):
                         continue
-                    parameters = itertools.chain(resolved_definition.get("parameters", ()), common_parameters)
+                    parameters = list(itertools.chain(resolved_definition.get("parameters", ()), common_parameters))
                     # To prevent recursion errors we need to pass not resolved schema as well
                     # It could be used for response validation
-                    raw_definition = EndpointDefinition(raw_methods[method], resolved_definition, scope)
+                    raw_definition = EndpointDefinition(raw_methods[method], resolved_definition, scope, parameters)
                     yield self.make_endpoint(path, method, parameters, resolved_definition, raw_definition)
         except (KeyError, AttributeError, jsonschema.exceptions.RefResolutionError) as exc:
             raise InvalidSchema("Schema parsing failed. Please check your schema.") from exc
@@ -82,11 +103,11 @@ class BaseOpenAPISchema(BaseSchema):
             return deepcopy(self.resolver.resolve(methods["$ref"]))
         return self.resolver.resolution_scope, deepcopy(methods)
 
-    def make_endpoint(  # pylint: disable=too-many-arguments
+    def make_endpoint(
         self,
         path: str,
         method: str,
-        parameters: Iterator[Dict[str, Any]],
+        parameters: List[Dict[str, Any]],
         resolved_definition: Dict[str, Any],
         raw_definition: EndpointDefinition,
     ) -> Endpoint:
@@ -94,7 +115,7 @@ class BaseOpenAPISchema(BaseSchema):
         base_url = self.get_base_url()
         endpoint = Endpoint(
             path=path,
-            method=method.upper(),
+            method=method,
             definition=raw_definition,
             base_url=base_url,
             app=self.app,
@@ -147,8 +168,8 @@ class BaseOpenAPISchema(BaseSchema):
             for method, resolved_definition in methods.items():
                 if method not in self.operations or "operationId" not in resolved_definition:
                     continue
-                parameters = itertools.chain(resolved_definition.get("parameters", ()), common_parameters)
-                raw_definition = EndpointDefinition(raw_methods[method], resolved_definition, scope)
+                parameters = list(itertools.chain(resolved_definition.get("parameters", ()), common_parameters))
+                raw_definition = EndpointDefinition(raw_methods[method], resolved_definition, scope, parameters)
                 yield resolved_definition["operationId"], self.make_endpoint(
                     path, method, parameters, resolved_definition, raw_definition
                 )
@@ -165,17 +186,26 @@ class BaseOpenAPISchema(BaseSchema):
         parent_ref, _ = reference.rsplit("/", maxsplit=1)
         _, methods = self.resolver.resolve(parent_ref)
         common_parameters = get_common_parameters(methods)
-        parameters = itertools.chain(resolved_definition.get("parameters", ()), common_parameters)
-        raw_definition = EndpointDefinition(data, resolved_definition, scope)
+        parameters = list(itertools.chain(resolved_definition.get("parameters", ()), common_parameters))
+        raw_definition = EndpointDefinition(data, resolved_definition, scope, parameters)
         return self.make_endpoint(path, method, parameters, resolved_definition, raw_definition)
 
     def get_case_strategy(
-        self, endpoint: Endpoint, hooks: Optional[HookDispatcher] = None, feedback: Optional[Feedback] = None
+        self,
+        endpoint: Endpoint,
+        hooks: Optional[HookDispatcher] = None,
+        feedback: Optional[Feedback] = None,
+        data_generation_method: DataGenerationMethod = DataGenerationMethod.default(),
     ) -> SearchStrategy:
-        return get_case_strategy(endpoint, hooks, feedback)
+        return get_case_strategy(endpoint, hooks, feedback, data_generation_method)
 
     def get_hypothesis_conversion(self, endpoint: Endpoint, location: str) -> Optional[Callable]:
         definitions = [item for item in endpoint.definition.resolved.get("parameters", []) if item["in"] == location]
+        security_parameters = self.security.get_security_definitions_as_parameters(
+            self.raw_schema, endpoint, self.resolver, location
+        )
+        if security_parameters:
+            definitions.extend(security_parameters)
         if definitions:
             return self._get_hypothesis_conversion(definitions)
         return None
@@ -183,6 +213,154 @@ class BaseOpenAPISchema(BaseSchema):
     def _get_hypothesis_conversion(self, definitions: List[Dict[str, Any]]) -> Optional[Callable]:
         raise NotImplementedError
 
+    def _get_response_definitions(self, endpoint: Endpoint, response: GenericResponse) -> Optional[Dict[str, Any]]:
+        try:
+            responses = endpoint.definition.resolved["responses"]
+        except KeyError as exc:
+            # Possible to get if `validate_schema=False` is passed during schema creation
+            raise InvalidSchema("Schema parsing failed. Please check your schema.") from exc
+        status_code = str(response.status_code)
+        if status_code in responses:
+            return responses[status_code]
+        if "default" in responses:
+            return responses["default"]
+        return None
+
+    def get_headers(self, endpoint: Endpoint, response: GenericResponse) -> Optional[Dict[str, Dict[str, Any]]]:
+        definitions = self._get_response_definitions(endpoint, response)
+        if not definitions:
+            return None
+        return definitions.get("headers")
+
+    def as_state_machine(self) -> Type[APIStateMachine]:
+        return create_state_machine(self)
+
+    def add_link(
+        self,
+        source: Endpoint,
+        target: Union[str, Endpoint],
+        status_code: Union[str, int],
+        parameters: Optional[Dict[str, str]] = None,
+        request_body: Any = None,
+    ) -> None:
+        """Add a new Open API link to the schema definition.
+
+        :param Endpoint source: This operation is the source of data
+        :param target: This operation will receive the data from this link.
+            Can be an ``Endpoint`` instance or a reference like this - ``#/paths/~1users~1{userId}/get``
+        :param str status_code: The link is triggered when the source endpoint responds with this status code.
+        :param parameters: A dictionary that describes how parameters should be extracted from the matched response.
+            The key represents the parameter name in the target endpoint, and the value is a runtime expression string.
+        :param request_body: A literal value or runtime expression to use as a request body when
+            calling the target operation.
+
+        .. code-block:: python
+
+            schema = schemathesis.from_uri("http://0.0.0.0/schema.yaml")
+
+            schema.add_link(
+                source=schema["/users/"]["POST"],
+                target=schema["/users/{userId}"]["GET"],
+                status_code="201",
+                parameters={"userId": "$response.body#/id"},
+            )
+        """
+        if parameters is None and request_body is None:
+            raise ValueError("You need to provide `parameters` or `request_body`.")
+        if hasattr(self, "_endpoints"):
+            delattr(self, "_endpoints")
+        for endpoint, methods in self.raw_schema["paths"].items():
+            if endpoint == source.path:
+                # Methods should be completely resolved now, otherwise they might miss a resolving scope when
+                # they will be fully resolved later
+                methods = self.resolver.resolve_all(methods)
+                found = False
+                for method, definition in methods.items():
+                    if method.upper() == source.method.upper():
+                        found = True
+                        links.add_link(
+                            definition["responses"], self.links_field, parameters, request_body, status_code, target
+                        )
+                    # If methods are behind a reference, then on the next resolving they will miss the new link
+                    # Therefore we need to modify it this way
+                    self.raw_schema["paths"][endpoint][method] = definition
+                # The reference should be removed completely, otherwise new keys in this dictionary will be ignored
+                # due to the `$ref` keyword behavior
+                self.raw_schema["paths"][endpoint].pop("$ref", None)
+                if found:
+                    return
+        message = f"No such endpoint: `{source.verbose_name}`."
+        possibilities = [e.verbose_name for e in self.get_all_endpoints()]
+        matches = get_close_matches(source.verbose_name, possibilities)
+        if matches:
+            message += f" Did you mean `{matches[0]}`?"
+        message += " Check if the requested endpoint passes the filters in the schema."
+        raise ValueError(message)
+
+    def get_links(self, endpoint: Endpoint) -> Dict[str, Dict[str, Any]]:
+        result: Dict[str, Dict[str, Any]] = defaultdict(dict)
+        for status_code, link in links.get_all_links(endpoint):
+            result[status_code][link.name] = link
+        return result
+
+    def validate_response(self, endpoint: Endpoint, response: GenericResponse) -> None:
+        responses = {str(key): value for key, value in endpoint.definition.raw.get("responses", {}).items()}
+        status_code = str(response.status_code)
+        if status_code in responses:
+            definition = responses[status_code]
+        elif "default" in responses:
+            definition = responses["default"]
+        else:
+            # No response defined for the received response status code
+            return
+        scopes, schema = self.get_response_schema(definition, endpoint.definition.scope)
+        if not schema:
+            # No schema to check against
+            return
+        content_type = response.headers.get("Content-Type")
+        if content_type is None:
+            media_types = "\n    ".join(self.get_content_types(endpoint, response))
+            raise get_missing_content_type_error()(
+                "The response is missing the `Content-Type` header. The schema defines the following media types:\n\n"
+                f"    {media_types}"
+            )
+        if not content_type.startswith("application/json"):
+            return
+        try:
+            if isinstance(response, requests.Response):
+                data = response.json()
+            else:
+                data = response.json
+        except JSONDecodeError as exc:
+            exc_class = get_response_parsing_error(exc)
+            payload = get_response_payload(response)
+            raise exc_class(
+                f"The received response is not valid JSON:\n\n    {payload}\n\nException: \n\n    {exc}"
+            ) from exc
+        with in_scopes(self.resolver, scopes):
+            try:
+                jsonschema.validate(data, schema, cls=jsonschema.Draft4Validator, resolver=self.resolver)
+            except jsonschema.ValidationError as exc:
+                exc_class = get_schema_validation_error(exc)
+                raise exc_class(
+                    f"The received response does not conform to the defined schema!\n\nDetails: \n\n{exc}"
+                ) from exc
+        return None  # explicitly return None for mypy
+
+
+@contextmanager
+def in_scopes(resolver: jsonschema.RefResolver, scopes: List[str]) -> Generator[None, None, None]:
+    """Push all available scopes into the resolver.
+
+    There could be an additional scope change during a schema resolving in `get_response_schema`, so in total there
+    could be a stack of two scopes maximum. This context manager handles both cases (1 or 2 scope changes) in the same
+    way.
+    """
+    with ExitStack() as stack:
+        for scope in scopes:
+            stack.enter_context(resolver.in_scope(scope))
+        yield
+
 
 class SwaggerV20(BaseOpenAPISchema):
     nullable_name = "x-nullable"
@@ -333,11 +511,11 @@ class OpenApi30(SwaggerV20):  # pylint: disable=too-many-ancestors
             return urlsplit(url).path
         return "/"
 
-    def make_endpoint(  # pylint: disable=too-many-arguments
+    def make_endpoint(
         self,
         path: str,
         method: str,
-        parameters: Iterator[Dict[str, Any]],
+        parameters: List[Dict[str, Any]],
         resolved_definition: Dict[str, Any],
         raw_definition: EndpointDefinition,
     ) -> Endpoint:
@@ -404,17 +582,8 @@ class OpenApi30(SwaggerV20):  # pylint: disable=too-many-ancestors
         return get_strategies_from_examples(endpoint, self.examples_field)
 
     def get_content_types(self, endpoint: Endpoint, response: GenericResponse) -> List[str]:
-        try:
-            responses = endpoint.definition.raw["responses"]
-        except KeyError as exc:
-            # Possible to get if `validate_schema=False` is passed during schema creation
-            raise InvalidSchema("Schema parsing failed. Please check your schema.") from exc
-        status_code = str(response.status_code)
-        if status_code in responses:
-            definitions = responses[status_code]
-        elif "default" in responses:
-            definitions = responses["default"]
-        else:
+        definitions = self._get_response_definitions(endpoint, response)
+        if not definitions:
             return []
         return list(definitions.get("content", {}).keys())
 
