@@ -4,7 +4,8 @@ import re
 import sys
 import traceback
 from contextlib import contextmanager
-from typing import Any, Callable, Dict, Generator, List, Optional, Set, Tuple, Type, Union, overload
+from json import JSONDecodeError
+from typing import Any, Callable, Dict, Generator, List, NoReturn, Optional, Set, Tuple, Type, Union, overload
 
 import requests
 import yaml
@@ -13,8 +14,8 @@ from requests.auth import HTTPDigestAuth
 from requests.exceptions import InvalidHeader  # type: ignore
 from requests.utils import check_header_validity  # type: ignore
 from werkzeug.wrappers import Response as BaseResponse
-from werkzeug.wrappers.json import JSONMixin
 
+from ._compat import JSONMixin
 from .types import Filter, NotSet, RawAuth
 
 try:
@@ -129,8 +130,11 @@ def format_exception(error: Exception, include_traceback: bool = False) -> str:
 
 def parse_content_type(content_type: str) -> Tuple[str, str]:
     """Parse Content Type and return main type and subtype."""
-    content_type, _ = cgi.parse_header(content_type)
-    main_type, sub_type = content_type.split("/", 1)
+    try:
+        content_type, _ = cgi.parse_header(content_type)
+        main_type, sub_type = content_type.split("/", 1)
+    except ValueError as exc:
+        raise ValueError(f"Malformed media type: `{content_type}`") from exc
     return main_type.lower(), sub_type.lower()
 
 
@@ -170,7 +174,12 @@ StringDatesYAMLLoader = make_loader("tag:yaml.org,2002:timestamp")
 
 
 class WSGIResponse(BaseResponse, JSONMixin):  # pylint: disable=too-many-ancestors
-    pass
+    # We store "requests" request to build a reproduction code
+    request: requests.PreparedRequest
+
+    def on_json_loading_failed(self, e: JSONDecodeError) -> NoReturn:
+        # We don't need a werkzeug-specific exception when JSON parsing error happens
+        raise e
 
 
 def get_requests_auth(auth: Optional[RawAuth], auth_type: Optional[str]) -> Optional[Union[HTTPDigestAuth, RawAuth]]:
@@ -182,9 +191,15 @@ def get_requests_auth(auth: Optional[RawAuth], auth_type: Optional[str]) -> Opti
 GenericResponse = Union[requests.Response, WSGIResponse]  # pragma: no mutate
 
 
+def get_response_payload(response: GenericResponse) -> str:
+    if isinstance(response, requests.Response):
+        return response.text
+    return response.get_data(as_text=True)
+
+
 def import_app(path: str) -> Any:
     """Import an application from a string."""
-    path, name = (re.split(r":(?![\\/])", path, 1) + [None])[:2]  # type: ignore
+    path, name = (re.split(r":(?![\\/])", path, 1) + [""])[:2]
     __import__(path)
     # accessing the module from sys.modules returns a proper module, while `__import__`
     # may return a parent module (system dependent)
